@@ -14,6 +14,7 @@
  * 23.9.2006, mild re-engineering for WO support --Sampo
  * 23.2.2008, added support for function pointers to malloc(),
  *            realloc(), and free() --Sampo
+ * 27.10.2010, namespace re-engineering --Sampo
  *
  * This file is included from various generated grammar files.
  */
@@ -42,33 +43,10 @@
 #define fdtype int
 #endif
 
-//#ifndef const
-//#define const  /* const causes swig java to break */
-//#endif
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-/* Error Codes (low level or XML parsing) */
-
-#define ZXERR_EOF             0x0001  /* return due to EOF condition */
-#define ZXERR_MISMATCH_CLOSE  0x0002  /* mismatching close tag */
-#define ZXERR_BAD_ATTR_NS     0x0004  /* namespace violation in attribute */
-#define ZXERR_BAD_ELEM_NS     0x0008  /* namespace violation in element */
-#define ZXERR_MISSING_ATTR    0x0010  /* mandatory attribute(s) missing */
-#define ZXERR_EXTRA_ATTR      0x0020  /* unforeseen attribute(s) present */
-#define ZXERR_MISSING_ELEM    0x0040  /* mandatory element(s) missing or of wrong cardinality */
-#define ZXERR_EXTRA_ELEM      0x0080  /* unforeseen element(s) present */
-#define ZXERR_ELEM_ORDER      0x0100  /* ordering of elements violates schema */
-#define ZXERR_BAD_NS_PREFIX   0x0200  /* unknown namespace prefix has appeared */
-#define ZXERR_TAG_NOT_CLOSED  0x0400  /* open tag seen, but eof seen before close */
-#define ZXERR_MASK            0x7fff
-#define ZXERR_ATTR_FLAG       0x8000  /* Not an error. Marker to distinguish elements from attributes. */
-
-struct zx_any_elem_s;
-struct zx_any_attr_s;
-
 /*(s) Namespace management. The context references this table. The array is
  * terminated by an element with empty URL (url_len == 0). The elements
  * of the array are the official namespace prefixes derived from
@@ -78,25 +56,24 @@ struct zx_any_attr_s;
  * prefixes of namespaces not understood by the system. */
 
 struct zx_ns_s {
+  /*int name;              / * For gperf -P (%pic) string-pool offset when in hash. */
+  const char* url;          /* Needs to be first so gperf (without -P or %pic) works */
+  int url_len;              /* 0 = end of nstab */
+  int prefix_len;
+  const char* prefix;
+  struct zx_el_tok* (*elem2tok)(const char* name, int len);  /* Pointer to the (xsd2sg + gperf generated) perfect hash function for determining tok in namespace */
+  struct zx_el_tok* el_tab; /* Table of element names (for perfoect hash). */
   struct zx_ns_s* n;        /* For holding runtime equivalences as a linked list. */
-  struct zx_ns_s* m;        /* For a rt equivalence, pointer to the master entry. */
+  struct zx_ns_s* master;   /* For a runtime equivalence, pointer to the master entry. */
   struct zx_ns_s* seen;     /* Pointer to other "seen" namespaces with same prefix (stack) */
   struct zx_ns_s* seen_n;   /* Next prefix in seen structure (list) */
   struct zx_ns_s* seen_p;   /* Previous prefix in seen structure (list) */
   struct zx_ns_s* seen_pop; /* Pop list for seen stack (used in the end of an element). */
   struct zx_ns_s* inc_n;    /* Next link for InclusiveNamespaces */
-#if 0
-  char flags;               /* 0x01 == unknown */
-  char pad1;
-  char pad2;
-  char pad3;
-#endif
-  int prefix_len;
-  char* prefix;
-  int url_len;  /* 0 = end of nstab */
-  char* url;
 };
 
+/*struct zx_ns_s zx_ns_tab[]; include c/zx-ns.h instead */
+
 /* Context tracks the input and namespaces. It is also passed to memory allocator. */
 
 struct zx_ctx {
@@ -104,6 +81,7 @@ struct zx_ctx {
   const char* p;     /* Current scan pointer */
   const char* lim;
   struct zx_ns_s* ns_tab;      /* Array, such as zx_ns_tab, see zx_prepare_dec_ctx() */
+  int n_ns;                    /* Number of entries in ns_tab. */
   struct zx_ns_s* unknown_ns;  /* Linked list of unknown namespaces. */
   /* Namespace prefixes that have been "seen", each prefix is a stack.
    * We keep these prefixes in a doubly linked list so we can add and
@@ -133,53 +111,74 @@ struct zx_ctx {
   char pad7;
 };
 
-/* We arrange all structs to start with a common header (16 bytes on 32bit platforms) */
+/* We arrange all structs to start with a common header (16 bytes on 32bit platforms).
+ * This structure works as a binary clean string. When used as (a part of) an
+ * element, the namespace prefix and name of the element form the string. The
+ * token value does not need to be represented as it can be recovered by
+ * performing zx_elem2tok() lookup again. The namespace information is
+ * implicit in the placement of the element in its parent element's struct. */
+/* All attributes are represented directly as a string. */
+/* *** idea: combine n & wo. Multioccurrance in wo would have to be detected
+ * *** by checking if the next is still of the same type. As multioccurances
+ * *** are rare, this would not hurt much. */
 
-struct zx_node_s {
-  struct zx_node_s* n;    /* next pointer for compile time construction of data structures */
-  struct zx_node_s* wo;   /* next pointer for canonicalization order (wire order) */
-  struct zx_ns_s* ns;     /* namespace of an element or attribute */
-  short tok;              /* token number of the tag represented by this struct */
-  short err;              /* error mask */
+//#define wo n
+
+struct zx_str {
+  struct zx_str* n;  /* next pointer for compile time construction of data structures */
+  int tok;           /* token number of the ns+tag represented by this struct */
+  int len;
+  char* s;           /* Start of prefix:element in the scan buffer. */
 };
 
 //#define ZX_NEXT(x) ((void*)((struct zx_node_s*)(x))->n)
 //#define ZX_NEXT(x) ((void*)((x)->gg.g.n))
 #define ZX_NEXT(x) ((x)->gg.g.n)
 
+/* Attributes that are unforeseen (errornous or extensions). */
+
+struct zx_attr_s {
+  struct zx_str g;
+  struct zx_ns_s* ns;        /* namespace of the attribute */
+  int name_len;
+  char* name;
+};
+
+  //#define ZX_ANY_AT(x) ((struct zx_any_attr_s*)(x))
+
 /* Simple elements, base type for complex elements. */
 
 struct zx_elem_s {
-  struct zx_node_s g;              /* Common fields for all nodes */
-  struct zx_elem_s* kids;          /* root of wo list representing child elements */
-  struct zx_any_attr_s* any_attr;  /* list of attributes not understood by parser */
-  struct zx_any_elem_s* any_elem;  /* list of elements not understood by parser */
-  struct zx_ns_s* xmlns;           /* xmlns declarations (for inc_ns processing) */
-  struct zx_str* content;          /* non-element content, if any */
+  struct zx_str g;             /* Common fields for all nodes */
+  struct zx_elem_s* kids;      /* root of wo list representing child elements */
+  struct zx_attr_s* attr;      /* list of attributes */
+  struct zx_ns_s*   ns;        /* namespace of the element */
+  struct zx_ns_s*   xmlns;     /* xmlns declarations (for inc_ns processing) */
 };
 
-struct zx_elem_s* zx_new_simple_elem(struct zx_ctx* c, struct zx_str* ss);
-struct zx_elem_s* zx_ref_len_simple_elem(struct zx_ctx* c, int len, const char* s);
-struct zx_elem_s* zx_ref_simple_elem(struct zx_ctx* c, const char* s);
-struct zx_elem_s* zx_dup_len_simple_elem(struct zx_ctx* c, int len, const char* s);
-struct zx_elem_s* zx_dup_simple_elem(struct zx_ctx* c, const char* s);
-#define ZX_SIMPLE_ELEM_CHK(el) ((el) && (el)->gg.content && (el)->gg.content->len && (el)->gg.content->s && (el)->gg.content->s[0])
+#define ZX_ELEM_EXT struct zx_elem_s gg;   /* Used in generated data types */
 
-/* All attributes are represented as a string, as follows. */
+struct zx_elem_s* zx_new_simple_elem(struct zx_ctx* c, struct zx_elem_s* father, int tok, struct zx_str* ss);
+struct zx_elem_s* zx_ref_len_simple_elem(struct zx_ctx* c, struct zx_elem_s* father, int tok, int len, const char* s);
+struct zx_elem_s* zx_ref_simple_elem(struct zx_ctx* c, struct zx_elem_s* father, int tok, const char* s);
+struct zx_elem_s* zx_dup_len_simple_elem(struct zx_ctx* c, struct zx_elem_s* father, int tok, int len, const char* s);
+struct zx_elem_s* zx_dup_simple_elem(struct zx_ctx* c, struct zx_elem_s* father, int tok, const char* s);
 
-struct zx_str {
-  struct zx_node_s g;
-  int len;
-  char* s;
-};
+struct zx_attr_s* zx_ref_len_attr(struct zx_ctx* c, int tok, int len, const char* s);
+struct zx_attr_s* zx_ref_attr(struct zx_ctx* c, int tok, const char* s);
+struct zx_attr_s* zx_new_len_attr(struct zx_ctx* c, int tok, int len);
+struct zx_attr_s* zx_dup_len_attr(struct zx_ctx* c, int tok, int len, const char* s);
+struct zx_attr_s* zx_dup_attr(struct zx_ctx* c, int tok, const char* s);
+struct zx_attr_s* zx_attrf(struct zx_ctx* c, int tok, const char* f, ...);
 
 struct zx_str* zx_ref_str(struct zx_ctx* c, const char* s);  /* ref points to underlying data */
 struct zx_str* zx_ref_len_str(struct zx_ctx* c, int len, const char* s);
 struct zx_str* zx_new_len_str(struct zx_ctx* c, int len);
-struct zx_str* zx_dup_str(struct zx_ctx* c, const char* s);  /* data is new memory */
 struct zx_str* zx_dup_len_str(struct zx_ctx* c, int len, const char* s);
-char* zx_alloc_sprintf(struct zx_ctx* c, int* retlen, const char* f, ...);
+struct zx_str* zx_dup_str(struct zx_ctx* c, const char* s);  /* data is new memory */
 struct zx_str* zx_strf(struct zx_ctx* c, const char* f, ...);  /* data is new memory */
+
+char* zx_alloc_sprintf(struct zx_ctx* c, int* retlen, const char* f, ...);
 void  zx_str_free(struct zx_ctx* c, struct zx_str* ss);   /* free both ss->s and ss */
 char* zx_str_to_c(struct zx_ctx* c, struct zx_str* ss);
 void  zx_str_conv(struct zx_str* ss, int* out_len, char** out_s);  /* SWIG typemap friendly */
@@ -187,29 +186,13 @@ int   zx_str_ends_in(struct zx_str* ss, int len, const char* suffix);
 #define ZX_STRCMP(a, b) ((a)?((b)?((a)->len == (b)->len?memcmp((a)->s, (b)->s, (a)->len):(a)->len - (b)->len):1):((b)?-1:0))
 #define ZX_STR_EQ(ss, cstr) ((ss) && (cstr) && (ss)->s && (ss)->len == strlen(cstr) && !memcmp((cstr), (ss)->s, (ss)->len))
 #define ZX_STR_ENDS_IN_CONST(ss, suffix) zx_str_ends_in((ss), sizeof(suffix)-1, (suffix))
-#define ZX_CONTENT_EQ_CONST(e, c) ((e) && (e)->content->len == sizeof(c)-1 && !memcmp((e)->content->s, (c), sizeof(c)-1))
 
-/* Elements that are unforeseen (errornous or extensions). */
-
-struct zx_any_elem_s {
-  struct zx_elem_s gg;
-  int name_len;
-  char* name;
-};
-
-#define ZX_ANY_EL(x) ((struct zx_any_elem_s*)(x))
-
-/* Attributes that are unforeseen (errornous or extensions). */
-
-struct zx_any_attr_s {
-  struct zx_str ss;
-  int name_len;
-  char* name;
-};
-
-#define ZX_ANY_AT(x) ((struct zx_any_attr_s*)(x))
-
-#define ZX_ELEM_EXT struct zx_elem_s gg;
+#define ZX_ELEM_S(e) ((struct zx_elem_s*)(e))
+#define ZX_SIMPLE_ELEM_CHK(e) ((e) && ZX_ELEM_S(e)->kids && ZX_ELEM_S(e)->kids->g.tok == ZX_TOK_DATA && ZX_ELEM_S(e)->kids->g.len && ZX_ELEM_S(e)->kids->g.s && ZX_ELEM_S(e)->kids->g.s[0])
+#define ZX_CONTENT_EQ_CONST(e, c) ((e) && ZX_ELEM_S(e)->kids && ZX_ELEM_S(e)->kids->g.tok == ZX_TOK_DATA && ZX_ELEM_S(e)->kids->g.len == sizeof(c)-1 && !memcmp(ZX_ELEM_S(e)->kids->g.s, (c), sizeof(c)-1))
+#define ZX_GET_CONTENT(e) ((e) && ZX_ELEM_S(e)->kids && ZX_ELEM_S(e)->kids->g.tok == ZX_TOK_DATA ? &ZX_ELEM_S(e)->kids->g : 0)
+#define ZX_GET_CONTENT_LEN(e) ((e) && ZX_ELEM_S(e)->kids && ZX_ELEM_S(e)->kids->g.tok == ZX_TOK_DATA ? ZX_ELEM_S(e)->kids->g.len : 0)
+#define ZX_GET_CONTENT_S(e) ((e) && ZX_ELEM_S(e)->kids && ZX_ELEM_S(e)->kids->g.tok == ZX_TOK_DATA ? ZX_ELEM_S(e)->kids->g.s : 0)
 
 char* zx_memmem(const char* haystack, int haystack_len, const char* needle, int needle_len);
 void* zx_alloc(struct zx_ctx* c, int size);
@@ -225,7 +208,8 @@ void  zx_reset_ctx(struct zx_ctx* ctx);
 struct zx_ctx* zx_init_ctx();   /* from malloc(3) */
 
 /* N.B. All string scanning assumes buffer is terminated with C string style nul byte. */
-#define ZX_SKIP_WS_P(c,p,x) MB for (; ONE_OF_4(*(p), ' ', '\n', '\r', '\t'); ++(p)) ; if (!*(p)) return x; ME
+/*#define ZX_SKIP_WS_P(c,p,x) MB for (; ONE_OF_4(*(p), ' ', '\n', '\r', '\t'); ++(p)) ; if (!*(p)) return x; ME*/
+#define ZX_SKIP_WS_P(c,p,x) MB p += strspn((p)," \n\r\t"); if (!*(p)) return x; ME
 #define ZX_SKIP_WS(c,x)     ZX_SKIP_WS_P((c),(c)->p,x)
 #define ZX_LOOK_FOR_P(c,ch,p) MB char* pp = memchr((p), (ch), (c)->lim - (p)); if (!pp) goto look_for_not_found; else (p) = pp; ME
 #define ZX_LOOK_FOR(c,ch)   ZX_LOOK_FOR_P((c),(ch),(c)->p)
@@ -246,10 +230,26 @@ struct zx_ctx* zx_init_ctx();   /* from malloc(3) */
 #endif
 
 /* Special token values. */
+#define ZX_TOK_NO_ATTR   (-7)
+#define ZX_TOK_ATTR_ERR  (-6)
 #define ZX_TOK_XMLNS     (-4)
-#define ZX_TOK_DATA      (-3)
-#define ZX_TOK_NOT_FOUND (-2)
-struct zx_tok { const char* name; const char* prefix; struct zx_ns_s* ns; };
+#define zx_root_ELEM            0x00fffffc
+#define ZX_TOK_DATA             0x0000fffd
+#define ZX_TOK_ATTR_NOT_FOUND   0x0000fffe
+#define ZX_TOK_NOT_FOUND        0x0000ffff
+#define ZX_TOK_NS_NOT_FOUND     0x00ff0000
+#define ZX_TOK_AND_NS_NOT_FOUND 0x00ffffff  /* Decimal 16777215 */
+#define ZX_TOK_TOK_MASK         0x0000ffff
+#define ZX_TOK_NS_MASK          0x00ff0000
+#define ZX_TOK_NS_SHIFT         16
+#define ZX_TOK_FLAGS_MASK       0xff000000
+
+struct zx_el_tok {
+  const char* name;
+  int siz;  /* struct size to help allocation */
+  //struct zx_elem_s* (*dec)(struct zx_ctx* c, struct zx_elem_s* x); /* funcptr to decode switch */
+};
+struct zx_at_tok { const char* name; };
 
 /*struct zx_note_s* zx_clone_any(struct zx_ctx* c, struct zx_note_s* n, int dup_strs); TBD */
 /*void zx_free_any(struct zx_ctx* c, struct zx_note_s* n, int free_strs); TBD */
@@ -258,16 +258,16 @@ int   zx_date_time_to_secs(const char* dt);
 int   write2_or_append_lock_c_path(const char* c_path, int len1, const char* data1, int len2, const char* data2, const char* which, int seeky, int flag);
 int   zx_report_openssl_error(const char* logkey);
 
-void  zx_fix_any_elem_dec(struct zx_ctx* c, struct zx_any_elem_s* x, const char* nam, int namlen);
-struct zx_ns_s* zx_new_ns(struct zx_ctx* c, int prefix_len, const char* prefix, int url_len, const char* url);
-struct zx_ns_s* zx_locate_ns_by_prefix(struct zx_ctx* c, int len, const char* prefix);
+#if 0
+void  zx_fix_any_elem_dec(struct zx_ctx* c, struct zx_elem_s* x, const char* nam, int namlen);
 int   zx_is_ns_prefix(struct zx_ns_s* ns, int len, const char* prefix);
+#endif
+int zx_dump_ns_tab(struct zx_ctx* c, int flags);
 struct zx_ns_s* zx_prefix_seen(struct zx_ctx* c, int len, const char* prefix);
 struct zx_ns_s* zx_prefix_seen_whine(struct zx_ctx* c, int len, const char* prefix, const char* logkey, int mk_dummy_ns);
 struct zx_ns_s* zx_scan_xmlns(struct zx_ctx* c);
 void  zx_see_elem_ns(struct zx_ctx* c, struct zx_ns_s** pop_seen, struct zx_elem_s* el);
 void  zx_pop_seen(struct zx_ns_s* ns);
-
 int zx_format_parse_error(struct zx_ctx* ctx, char* buf, int siz, char* logkey);
 
 /* zxcrypto.c - Glue to OpenSSL low level */
@@ -284,23 +284,22 @@ char* zx_md5_crypt(const char* pw, const char* salt, char* buf);
 
 /* Common Subexpression Elimination (CSE) for generated code. */
 
-const struct zx_tok* zx_tok_by_ns(const struct zx_tok* zt, const struct zx_tok* lim, int len, const char* name, struct zx_ns_s* ns);
 int   zx_len_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_s** pop_seen);
-char* zx_enc_xmlns_if_not_seen(struct zx_ctx* c, char* p, struct zx_ns_s* ns, struct zx_ns_s** pop_seen);
 void  zx_add_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_s** pop_seen);
 char* zx_enc_seen(char* p, struct zx_ns_s* ns);
-int   zx_len_so_common(struct zx_ctx* c, struct zx_elem_s* x);
-int   zx_len_wo_common(struct zx_ctx* c, struct zx_elem_s* x);
-char* zx_enc_unknown_attrs(char* p, struct zx_any_attr_s* aa);
+int   zx_len_so_common(struct zx_ctx* c, struct zx_elem_s* x, struct zx_ns_s** pop_seenp);
+void  zx_see_attr_ns(struct zx_ctx* c, struct zx_attr_s* aa, struct zx_ns_s** pop_seenp);
 char* zx_enc_so_unknown_elems_and_content(struct zx_ctx* c, char* p, struct zx_elem_s* x);
+int   zx_LEN_WO_any_elem(struct zx_ctx* c, struct zx_elem_s* x);
+char* zx_ENC_WO_any_elem(struct zx_ctx* c, struct zx_elem_s* x, char* p);
+struct zx_str* zx_EASY_ENC_WO_any_elem(struct zx_ctx* c, struct zx_elem_s* x);
 struct zx_str* zx_easy_enc_common(struct zx_ctx* c, char* p, char* buf, int len);
-int   zx_attr_so_len(struct zx_str* attr, int name_size);
-char* zx_attr_so_enc(char* p, struct zx_str* attr, char* name, int name_len);
-int   zx_attr_wo_len(struct zx_str* attr, int name_size);
-char* zx_attr_wo_enc(char* p, struct zx_str* attr, char* name, int name_len);
-void  zx_free_attr(struct zx_ctx* c, struct zx_str* attr, int free_strs);
-void  zx_free_elem_common(struct zx_ctx* c, struct zx_elem_s* x, int free_strs);
-void  zx_free_simple_elems(struct zx_ctx* c, struct zx_elem_s* se, int free_strs);
+int   zx_attr_so_len(struct zx_ctx* c, struct zx_attr_s* attr, int name_len, struct zx_ns_s** pop_seenp);
+char* zx_attr_so_enc(char* p, struct zx_attr_s* attr, char* name, int name_len);
+char* zx_attr_wo_enc(char* p, struct zx_attr_s* attr);
+void  zx_free_attr(struct zx_ctx* c, struct zx_attr_s* attr, int free_strs);
+void  zx_free_elem(struct zx_ctx* c, struct zx_elem_s* x, int free_strs);
+void  zx_add_content(struct zx_ctx* c, struct zx_elem_s* x, struct zx_str* cont);
 
 #ifdef ZX_ENA_AUX
 void  zx_dup_attr(struct zx_ctx* c, struct zx_str* attr);
@@ -314,26 +313,28 @@ int   zx_walk_so_simple_elems(struct zx_ctx* c, struct zx_elem_s* se, void* ctx,
 void  zx_dup_strs_simple_elems(struct zx_ctx* c, struct zx_elem_s* se);
 #endif
 
-void  zx_prepare_dec_ctx(struct zx_ctx* c, struct zx_ns_s* ns_tab, const char* start, const char* lim);
+void  zx_prepare_dec_ctx(struct zx_ctx* c, struct zx_ns_s* ns_tab, int n_ns, const char* start, const char* lim);
+struct zx_root_s* zx_dec_zx_root(struct zx_ctx* c, int len, const char* start, const char* func);
 int   zx_scan_data(struct zx_ctx* c, struct zx_elem_s* el);
 int   zx_scan_pi_or_comment(struct zx_ctx* c);
-struct zx_str* zx_dec_unknown_attr(struct zx_ctx* c, struct zx_elem_s* el, const char* name, const char* data, int tok, int ctx_tok);
-char* zx_dec_attr_val(struct zx_ctx* c, const char** name);
+void zx_known_attr_wrong_context(struct zx_ctx* c, struct zx_elem_s* x);
+void zx_known_elem_wrong_context(struct zx_ctx* c, struct zx_elem_s* x);
+const char* zx_dec_attr_val(struct zx_ctx* c, const char* func);
+void zx_dec_reverse_lists(struct zx_elem_s* x);
 void  zx_xml_parse_err(struct zx_ctx* c, char quote, const char* func, const char* msg);
+void  zx_xml_parse_dbg(struct zx_ctx* c, char quote, const char* func, const char* msg);
+const char* zx_scan_elem_start(struct zx_ctx* c, const char* func);
+int zx_scan_elem_end(struct zx_ctx* c, const char* start, const char* func);
+struct zx_ns_s* zx_xmlns_detected(struct zx_ctx* c, struct zx_elem_s* x, const char* data);
 
 int   zx_len_inc_ns(struct zx_ctx* c, struct zx_ns_s** pop_seenp);
 void  zx_add_inc_ns(struct zx_ctx* c, struct zx_ns_s** pop_seenp);
-char* zx_enc_inc_ns(struct zx_ctx* c, char* p, struct zx_ns_s** pop_seenp);
+int   zx_in_inc_ns(struct zx_ctx* c, struct zx_ns_s* new_ns);
 
-#if 0
-#define ZX_DEC_TAG_NOT_YET_CLOSED(x) (x).err |= ZXERR_TAG_NOT_CLOSED
-#define ZX_DEC_TAG_NOW_CLOSED(x)     (x).err &= ~ZXERR_TAG_NOT_CLOSED
-#define ZX_DEC_TAG_MISMATCH_CLOSE(x) (x).err |= ZXERR_MISMATCH_CLOSE
-#else
-#define ZX_DEC_TAG_NOT_YET_CLOSED(x)
-#define ZX_DEC_TAG_NOW_CLOSED(x)
-#define ZX_DEC_TAG_MISMATCH_CLOSE(x)
-#endif
+/* These do not have their own elems, so the table andlookup were not generated. Supply dummies. */
+#define zx_xs__ELEM_MAX 1
+#define zx_xsi__ELEM_MAX 1
+  //#define zx_xml__ELEM_MAX 1
 
 #define SIG_ALGO_RSA_SHA1  "http://www.w3.org/2000/09/xmldsig#rsa-sha1"
 #define SIG_ALGO_DSA_SHA1  "http://www.w3.org/2000/09/xmldsig#dsa-sha1"
