@@ -680,7 +680,7 @@ zxid_entity* zxid_get_ent_from_cache(zxid_conf* cf, struct zx_str* eid);
 int zxid_write_ent_to_cache(zxid_conf* cf, zxid_entity* ent);
 zxid_entity* zxid_parse_meta(zxid_conf* cf, char** md, char* lim);
 zxid_entity* zxid_get_meta_ss(zxid_conf* cf, struct zx_str* url);
-zxid_entity* zxid_get_meta(zxid_conf* cf, char* url);
+zxid_entity* zxid_get_meta(zxid_conf* cf, const char* url);
 zxid_entity* zxid_get_ent_ss(zxid_conf* cf, struct zx_str* eid);
 zxid_entity* zxid_get_ent(zxid_conf* cf, char* eid);
 zxid_entity* zxid_get_ent_by_succinct_id(zxid_conf* cf, char* raw_succinct_id);
@@ -688,20 +688,21 @@ zxid_entity* zxid_get_ent_by_sha1_name(zxid_conf* cf, char* sha1_name);
 zxid_entity* zxid_load_cot_cache(zxid_conf* cf);
 
 #ifdef USE_OPENSSL
-struct zx_ds_KeyInfo_s* zxid_key_info(zxid_conf* cf, X509* x);
-struct zx_md_KeyDescriptor_s* zxid_key_desc(zxid_conf* cf, char* use, X509* cert);
+struct zx_ds_KeyInfo_s* zxid_key_info(zxid_conf* cf, struct zx_elem_s* father, X509* x);
+struct zx_md_KeyDescriptor_s* zxid_key_desc(zxid_conf* cf, struct zx_elem_s* father, char* use, X509* cert);
 #endif
-struct zx_md_ArtifactResolutionService_s* zxid_ar_desc(zxid_conf* cf, char* binding, char* loc, char* resp_loc);
-struct zx_md_SingleSignOnService_s* zxid_sso_desc(zxid_conf* cf, char* binding, char* loc, char* resp_loc);
-struct zx_md_SingleLogoutService_s* zxid_slo_desc(zxid_conf* cf, char* binding, char* loc, char* resp_loc);
-struct zx_md_ManageNameIDService_s* zxid_mni_desc(zxid_conf* cf, char* binding, char* loc, char* resp_loc);
-struct zx_md_AssertionConsumerService_s* zxid_ac_desc(zxid_conf* cf, char* binding, char* loc, char* index);
-struct zx_md_IDPSSODescriptor_s* zxid_idp_sso_desc(zxid_conf* cf);
-struct zx_md_SPSSODescriptor_s* zxid_sp_sso_desc(zxid_conf* cf);
+struct zx_md_ArtifactResolutionService_s* zxid_ar_desc(zxid_conf* cf, struct zx_elem_s* father, char* binding, char* loc, char* resp_loc);
+struct zx_md_SingleSignOnService_s* zxid_sso_desc(zxid_conf* cf, struct zx_elem_s* father, char* binding, char* loc, char* resp_loc);
+struct zx_md_SingleLogoutService_s* zxid_slo_desc(zxid_conf* cf, struct zx_elem_s* father, char* binding, char* loc, char* resp_loc);
+struct zx_md_ManageNameIDService_s* zxid_mni_desc(zxid_conf* cf, struct zx_elem_s* father, char* binding, char* loc, char* resp_loc);
+struct zx_md_AssertionConsumerService_s* zxid_ac_desc(zxid_conf* cf, struct zx_elem_s* father, char* binding, char* loc, char* index);
+struct zx_md_IDPSSODescriptor_s* zxid_idp_sso_desc(zxid_conf* cf, struct zx_elem_s* father);
+struct zx_md_SPSSODescriptor_s* zxid_sp_sso_desc(zxid_conf* cf, struct zx_elem_s* father);
 struct zx_str* zxid_sp_meta(zxid_conf* cf, zxid_cgi* cgi);
 int zxid_send_sp_meta(zxid_conf* cf, zxid_cgi* cgi);
 struct zx_str* zxid_sp_carml(zxid_conf* cf);
 struct zx_str* zxid_my_entity_id(zxid_conf* cf);
+struct zx_attr_s* zxid_my_entity_id_attr(zxid_conf* cf, int tok);
 struct zx_str* zxid_my_cdc_url(zxid_conf* cf);
 struct zx_sa_Issuer_s* zxid_my_issuer(zxid_conf* cf);
 
@@ -774,9 +775,13 @@ int zxid_pw_authn(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses);
 
 int   zxid_version();
 char* zxid_version_str();
+
 struct zx_str* zxid_date_time(zxid_conf* cf, time_t secs);
 struct zx_str* zxid_mk_id(zxid_conf* cf, char* prefix, int bits); /* pseudo random ident. */
 
+struct zx_attr_s* zxid_date_time_attr(zxid_conf* cf, int tok, time_t secs);
+struct zx_attr_s* zxid_mk_id_attr(zxid_conf* cf, int tok, char* prefix, int bits);
+
 struct zx_str* zxid_http_post_raw(zxid_conf* cf, int url_len, const char* url, int len, const char* data);
 struct zx_root_s* zxid_soap_call_raw(zxid_conf* cf, struct zx_str* url, struct zx_str* data, char** ret_enve);
 struct zx_str* zxid_lecp_check(zxid_conf* cf, zxid_cgi* cgi);
