@@ -25,6 +25,7 @@ import javax.servlet.*;
 import javax.servlet.http.*;
 import java.util.regex.Pattern;
 import java.util.regex.Matcher;
+import java.util.Enumeration;
 
 public class zxidappdemo extends HttpServlet {
     static final boolean verbose = false;
@@ -153,12 +154,12 @@ public class zxidappdemo extends HttpServlet {
 
 	// Render logout buttons (optional)
 
-	out.print("[<a href=\"sso?gl=1&s="+ses.getValue("sesid")+"\">Local Logout</a> | <a href=\"sso?gr=1&s="+ses.getValue("sesid")+"\">Single Logout</a>]\n");
+	out.print("[<a href=\"sso?gl=1&s="+ses.getAttribute("sesid")+"\">Local Logout</a> | <a href=\"sso?gr=1&s="+ses.getAttribute("sesid")+"\">Single Logout</a>]\n");
 
 	// The SSO servlet will have done one iteration of authorization. The following
 	// serves to illustrate, how to explicitly call a PDP from your code.
 
-	if (zxidjni.az_cf(cf, "Action=Show", ses.getValue("sesid").toString()) == null) {
+	if (zxidjni.az_cf(cf, "Action=Show", ses.getAttribute("sesid").toString()) == null) {
 	    out.print("<p><b>Denied.</b> Normally page would not be shown, but we show the session attributes for debugging purposes.\n");
 	    //res.setStatus(302, "Denied");
 	} else {
@@ -172,21 +173,22 @@ public class zxidappdemo extends HttpServlet {
 	// Render protected content page (your application starts working)
 
 	out.print("<h4>HttpSession dump:</h4>");
-	String[] val_names = ses.getValueNames();
-	for (int i = 0; i < val_names.length; ++i) {
-	    if (val_names[i].equals("cn")
-		|| val_names[i].equals("role")
-		|| val_names[i].equals("o")
-		|| val_names[i].equals("ou")
-		|| val_names[i].equals("idpnid")
-		|| val_names[i].equals("nidfmt")
-		|| val_names[i].equals("affid")
-		|| val_names[i].equals("eid")
-		|| val_names[i].equals("urn:liberty:disco:2006-08:DiscoveryEPR")) {
-		out.print("<b>" + val_names[i] + "</b>: " + ses.getValue(val_names[i]) + "<br>\n");
+	Enumeration val_names = ses.getAttributeNames();
+	while (val_names.hasMoreElements()) {
+	    String name = (String)val_names.nextElement();
+	    if (name.equals("cn")
+		|| name.equals("role")
+		|| name.equals("o")
+		|| name.equals("ou")
+		|| name.equals("idpnid")
+		|| name.equals("nidfmt")
+		|| name.equals("affid")
+		|| name.equals("eid")
+		|| name.equals("urn:liberty:disco:2006-08:DiscoveryEPR")) {
+		out.print("<b>" + name + "</b>: " + ses.getAttribute(name) + "<br>\n");
 	    } else {
 		if (verbose)
-		    out.print(val_names[i] + ": " + ses.getValue(val_names[i]) + "<br>\n");
+		    out.print(name + ": " + ses.getAttribute(name) + "<br>\n");
 	    }
 	}
 	out.print("<p>");
@@ -201,7 +203,7 @@ public class zxidappdemo extends HttpServlet {
 	// Demo web service call to zxidhrxmlwsp
 
 	String ret;
-	String sid = ses.getValue("sesid").toString();
+	String sid = ses.getAttribute("sesid").toString();
 	zxid_ses zxses = zxidjni.fetch_ses(cf, sid);
 	
 	if (qs.equals("idhrxml") || qs.equals("all")) {
