@@ -79,6 +79,20 @@ struct zx_str* zxid_mk_id(zxid_conf* cf, char* prefix, int bits)
   return zx_strf(cf->ctx, "%s%.*s", prefix?prefix:"", p-base64_buf, base64_buf);
 }
 
+struct zx_attr_s* zxid_mk_id_attr(zxid_conf* cf, int tok, char* prefix, int bits)
+{
+  char bit_buf[ZXID_ID_MAX_BITS/8];
+  char base64_buf[ZXID_ID_MAX_BITS/6 + 1];
+  char* p;
+  if (bits > ZXID_ID_MAX_BITS || bits & 0x07) {
+    ERR("Requested bits(%d) more than internal limit(%d), or bits not divisible by 8.", bits, ZXID_ID_MAX_BITS);
+    return 0;
+  }
+  zx_rand(bit_buf, bits >> 3);
+  p = base64_fancy_raw(bit_buf, bits >> 3, base64_buf, safe_basis_64, 1<<31, 0, 0, '.');
+  return zx_attrf(cf->ctx, tok, "%s%.*s", prefix?prefix:"", p-base64_buf, base64_buf);
+}
+
 /*() Format a date-time string as usually used in XML, SAML, and Liberty. Apparently
  * there are two ways to format this: with or with-out milliseconds. ZXID accepts
  * either form as input, as they are both legal, but will only generate the
@@ -103,6 +117,22 @@ struct zx_str* zxid_date_time(zxid_conf* cf, time_t secs)
 #endif
 }
 
+struct zx_attr_s* zxid_date_time_attr(zxid_conf* cf, int tok, time_t secs)
+{
+  struct tm t;
+  secs += cf->timeskew;
+  GMTIME_R(secs, t);
+#if 0
+  /*                            "2002-10-31T21:42:14.002Z" */
+  return zx_attrf(cf->ctx, tok, "%04d-%02d-%02dT%02d:%02d:%02d.002Z",
+		  t.tm_year+1900, t.tm_mon+1, t.tm_mday, t.tm_hour, t.tm_min, t.tm_sec);
+#else
+  /*                            "2002-10-31T21:42:14Z" */
+  return zx_attrf(cf->ctx, tok, "%04d-%02d-%02dT%02d:%02d:%02dZ",
+		  t.tm_year+1900, t.tm_mon+1, t.tm_mday, t.tm_hour, t.tm_min, t.tm_sec);
+#endif
+}
+
 /* ============== SOAP Call ============= */
 
 /*() Encode XML data structure representing SOAP envelope (request)
@@ -146,7 +176,7 @@ struct zx_root_s* zxid_soap_call_hdr_body(zxid_conf* cf, struct zx_str* url, str
 {
   struct zx_root_s* r;
   struct zx_str* ss;
-  struct zx_e_Envelope_s* env = zx_NEW_e_Envelope(cf->ctx);
+  struct zx_e_Envelope_s* env = zx_NEW_e_Envelope(cf->ctx,0);
   env->Header = hdr;
   env->Body = body;
   ss = zx_EASY_ENC_SO_e_Envelope(cf->ctx, env);
@@ -169,7 +199,7 @@ struct zx_root_s* zxid_soap_call_hdr_body(zxid_conf* cf, struct zx_str* url, str
 /* Called by:  zxid_as_call_ses, zxid_idp_soap, zxid_sp_deref_art, zxid_sp_soap */
 struct zx_root_s* zxid_soap_call_body(zxid_conf* cf, struct zx_str* url, struct zx_e_Body_s* body)
 {
-  /*return zxid_soap_call_hdr_body(cf, url, zx_NEW_e_Header(cf->ctx), body);*/
+  /*return zxid_soap_call_hdr_body(cf, url, zx_NEW_e_Header(cf->ctx,0), body);*/
   return zxid_soap_call_hdr_body(cf, url, 0, body);
 }
 
@@ -184,11 +214,11 @@ struct zx_root_s* zxid_soap_call_body(zxid_conf* cf, struct zx_str* url, struct
 /* Called by:  zxid_idp_soap_dispatch x2, zxid_sp_soap_dispatch x6 */
 int zxid_soap_cgi_resp_body(zxid_conf* cf, struct zx_e_Body_s* body, struct zx_str* entid)
 {
-  struct zx_e_Envelope_s* env = zx_NEW_e_Envelope(cf->ctx);
+  struct zx_e_Envelope_s* env = zx_NEW_e_Envelope(cf->ctx,0);
   struct zx_str* ss;
   struct zx_str* logpath;
   
-  env->Header = zx_NEW_e_Header(cf->ctx);
+  env->Header = zx_NEW_e_Header(cf->ctx, &env->gg);
   env->Body = body;
   ss = zx_EASY_ENC_SO_e_Envelope(cf->ctx, env);
 
@@ -373,7 +403,7 @@ struct zx_str* zxid_saml2_post_enc(zxid_conf* cf, char* field, struct zx_str* pa
   return payload;
 }
 
-struct zx_str zxstr_unknown = { {0,0,0,0,0}, sizeof("UNKNOWN")-1, "UNKNOWN" };
+struct zx_str zxstr_unknown = {0,0,sizeof("UNKNOWN")-1, "UNKNOWN"};
 
 /*(i) Encode and sign a URL according to SAML2 redirect binding.
  * zxid_decode_redir_or_post() performs the opposite operation.
@@ -581,20 +611,20 @@ int zxid_saml_ok(zxid_conf* cf, zxid_cgi* cgi, struct zx_sp_Status_s* st, char*
   struct zx_str* sc1 = 0;
   struct zx_str* sc2 = 0;
   struct zx_sp_StatusCode_s* sc = st->StatusCode;
-  if (!memcmp(SAML2_SC_SUCCESS, sc->Value->s, sc->Value->len)) {
+  if (!memcmp(SAML2_SC_SUCCESS, sc->Value->g.s, sc->Value->g.len)) {
     D("SAML ok what(%s)", what);
     if (cf->log_level>0)
       zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "K", "SAMLOK", what, 0);
     return 1;
   }
-  if (st->StatusMessage && (m = st->StatusMessage->content))
+  if (st->StatusMessage && (m = ZX_GET_CONTENT(st->StatusMessage)))
     ERR("SAML Fail what(%s) msg(%.*s)", what, m->len, m->s);
-  if (sc1 = sc->Value)
+  if (sc1 = &sc->Value->g)
     ERR("SAML Fail what(%s) SC1(%.*s)", what, sc1->len, sc1->s);
   if (sc->StatusCode)
-    sc2 = sc->StatusCode->Value;
+    sc2 = &sc->StatusCode->Value->g;
   for (sc = sc->StatusCode; sc; sc = sc->StatusCode)
-    ERR("SAML Fail what(%s) subcode(%.*s)", what, sc->Value->len, sc->Value->s);
+    ERR("SAML Fail what(%s) subcode(%.*s)", what, sc->Value->g.len, sc->Value->g.s);
     
   ss = zx_strf(cf->ctx, "SAML Fail what(%s) msg(%.*s) SC1(%.*s) subcode(%.*s)", what,
 	       m?m->len:0, m?m->s:"",
@@ -638,10 +668,7 @@ zxid_nid* zxid_decrypt_nameid(zxid_conf* cf, zxid_nid* nid, struct zx_sa_Encrypt
       ERR("Failed to decrypt NameID. Most probably certificate-private key mismatch or metadata problem. Could also be corrupt message. %d", 0);
       return 0;
     }
-    LOCK(cf->ctx->mx, "dec nid");
-    zx_prepare_dec_ctx(cf->ctx, zx_ns_tab, ss->s, ss->s + ss->len);
-    r = zx_DEC_root(cf->ctx, 0, 1);
-    UNLOCK(cf->ctx->mx, "dec nid");
+    r = zx_dec_zx_root(cf->ctx, ss->len, ss->s, "dec nid");
     if (!r) {
       ERR("Failed to parse EncryptedID buf(%.*s)", ss->len, ss->s);
       return 0;
@@ -670,15 +697,12 @@ struct zx_str* zxid_decrypt_newnym(zxid_conf* cf, struct zx_str* newnym, struct
     return newnym;
   if (encid) {
     ss = zxenc_privkey_dec(cf, encid->EncryptedData, encid->EncryptedKey);
-    LOCK(cf->ctx->mx, "dec newnym");
-    zx_prepare_dec_ctx(cf->ctx, zx_ns_tab, ss->s, ss->s + ss->len);
-    r = zx_DEC_root(cf->ctx, 0, 1);
-    UNLOCK(cf->ctx->mx, "dec newnym");
+    r = zx_dec_zx_root(cf->ctx, ss->len, ss->s, "dec newnym");
     if (!r) {
       ERR("Failed to parse NewEncryptedID buf(%.*s)", ss->len, ss->s);
       return 0;
     }
-    return r->NewID->content;
+    return ZX_GET_CONTENT(r->NewID);
   }
   ERR("Neither NewNameID nor NewEncryptedID available %d", 0);
   return 0;
@@ -728,7 +752,7 @@ int zxid_chk_sig(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, struct zx_elem_s*
     goto erro;
   }
 
-  if (!issue_ent || !(issuer = issue_ent->gg.content) || !issuer->len || !issuer->s[0]) {
+  if (!issue_ent || !(issuer = ZX_GET_CONTENT(issue_ent)) || !issuer->len || !issuer->s[0]) {
     ERR("Issuer of %s is empty although %s was signed. %p", lk, lk, issuer);
     cgi->sigval = "I";
     cgi->sigmsg = "Issuer of signed Response missing.";
@@ -860,8 +884,8 @@ nobody:
   if (!p)
     goto nobody;
   
-  for (p += 4; *p && *p != '>'; ++p) ;  /* Scan for close of opening <Body */
-  if (!*p)
+  p = strchr(p+4, '>');  /* Scan for close of opening <Body */
+  if (!p)
     goto nobody;
   
   for (q = ++p; q; q+=5) {
