@@ -27,46 +27,46 @@
 struct zx_sp_AuthnRequest_s* zxid_mk_authn_req(zxid_conf* cf, zxid_cgi* cgi)
 {
   char index[2] = "1";
-  struct zx_sp_AuthnRequest_s* ar = zx_NEW_sp_AuthnRequest(cf->ctx);
+  struct zx_sp_AuthnRequest_s* ar = zx_NEW_sp_AuthnRequest(cf->ctx,0);
   ar->Issuer = zxid_my_issuer(cf);
-  ar->ID = zxid_mk_id(cf, "N", ZXID_ID_BITS);
-  ar->Version = zx_ref_str(cf->ctx, SAML2_VERSION);
-  ar->IssueInstant = zxid_date_time(cf, time(0));
-  if (cf->nice_name && cf->nice_name[0])  ar->ProviderName = zx_ref_str(cf->ctx, cf->nice_name);
-  if (BOOL_STR_TEST(cgi->force_authn))    ar->ForceAuthn = zx_ref_str(cf->ctx, ZXID_TRUE);
-  if (BOOL_STR_TEST(cgi->ispassive))      ar->IsPassive = zx_ref_str(cf->ctx, ZXID_TRUE);
-  if (cgi->consent && cgi->consent[0])    ar->Consent = zx_ref_str(cf->ctx, cgi->consent);
+  ar->ID = zxid_mk_id_attr(cf, zx_ID_ATTR, "N", ZXID_ID_BITS);
+  ar->Version = zx_ref_attr(cf->ctx, zx_Version_ATTR, SAML2_VERSION);
+  ar->IssueInstant = zxid_date_time_attr(cf, zx_IssueInstant_ATTR, time(0));
+  if (cf->nice_name && cf->nice_name[0])  ar->ProviderName = zx_ref_attr(cf->ctx, zx_ProviderName_ATTR, cf->nice_name);
+  if (BOOL_STR_TEST(cgi->force_authn))    ar->ForceAuthn = zx_ref_attr(cf->ctx, zx_ForceAuthn_ATTR, ZXID_TRUE);
+  if (BOOL_STR_TEST(cgi->ispassive))      ar->IsPassive = zx_ref_attr(cf->ctx, zx_IsPassive_ATTR, ZXID_TRUE);
+  if (cgi->consent && cgi->consent[0])    ar->Consent = zx_ref_attr(cf->ctx, zx_Consent_ATTR, cgi->consent);
   D("nid_fmt(%s) allow_create=%c ispassive=%c", cgi->nid_fmt, cgi->allow_create, cgi->ispassive);
   if (cgi->nid_fmt && cgi->nid_fmt[0] || cgi->affil && cgi->affil[0]
       || BOOL_STR_TEST(cgi->allow_create)) {
-    ar->NameIDPolicy = zx_NEW_sp_NameIDPolicy(cf->ctx);
+    ar->NameIDPolicy = zx_NEW_sp_NameIDPolicy(cf->ctx, &ar->gg);
     if (cgi->nid_fmt && cgi->nid_fmt[0])
-      ar->NameIDPolicy->Format = zx_ref_str(cf->ctx, zxid_saml2_map_nid_fmt(cgi->nid_fmt));
+      ar->NameIDPolicy->Format = zx_ref_attr(cf->ctx, zx_Format_ATTR, zxid_saml2_map_nid_fmt(cgi->nid_fmt));
     if (cgi->affil && cgi->affil[0])
-      ar->NameIDPolicy->SPNameQualifier = zx_ref_str(cf->ctx, cgi->affil);
+      ar->NameIDPolicy->SPNameQualifier = zx_ref_attr(cf->ctx, zx_SPNameQualifier_ATTR, cgi->affil);
     if (BOOL_STR_TEST(cgi->allow_create))
-      ar->NameIDPolicy->AllowCreate = zx_ref_str(cf->ctx, ZXID_TRUE);  /* default false */
+      ar->NameIDPolicy->AllowCreate = zx_ref_attr(cf->ctx, zx_AllowCreate_ATTR, ZXID_TRUE);  /* default false */
   }
   if (cgi->authn_ctx && cgi->authn_ctx[0]) {
-    ar->RequestedAuthnContext = zx_NEW_sp_RequestedAuthnContext(cf->ctx);
+    ar->RequestedAuthnContext = zx_NEW_sp_RequestedAuthnContext(cf->ctx, &ar->gg);
     ar->RequestedAuthnContext->AuthnContextClassRef
-      = zx_ref_simple_elem(cf->ctx, zxid_saml2_map_authn_ctx(cgi->authn_ctx));
+      = zx_ref_simple_elem(cf->ctx, &ar->RequestedAuthnContext->gg, zx_sa_AuthnContextClassRef_ELEM, zxid_saml2_map_authn_ctx(cgi->authn_ctx));
     if (cgi->matching_rule && cgi->matching_rule[0])
-      ar->RequestedAuthnContext->Comparison = zx_ref_str(cf->ctx, cgi->matching_rule);
+      ar->RequestedAuthnContext->Comparison = zx_ref_attr(cf->ctx, zx_Comparison_ATTR, cgi->matching_rule);
   }
   if (cgi->pr_ix && cgi->pr_ix != '0') {
     index[0] = cgi->pr_ix;
-    ar->AssertionConsumerServiceIndex = zx_dup_str(cf->ctx, index);
+    ar->AssertionConsumerServiceIndex = zx_dup_attr(cf->ctx, zx_AssertionConsumerServiceIndex_ATTR, index);
   }
   if (cgi->get_complete && cgi->get_complete[0]
       || cgi->pxy_count && cgi->pxy_count[0]
       || cgi->idppxylist && cgi->idppxylist[0]) {
-    ar->Scoping = zx_NEW_sp_Scoping(cf->ctx);
+    ar->Scoping = zx_NEW_sp_Scoping(cf->ctx, &ar->gg);
     if (cgi->pxy_count && cgi->pxy_count[0])
-      ar->Scoping->ProxyCount = zx_ref_str(cf->ctx, cgi->pxy_count);
+      ar->Scoping->ProxyCount = zx_ref_attr(cf->ctx, zx_ProxyCount_ATTR, cgi->pxy_count);
 #if 0
     if (cgi->get_complete && cgi->get_complete[0] || cgi->idppxylist) {
-      ar->Scoping->IDPList = zx_NEW_sp_IDPList(cf->ctx);
+      ar->Scoping->IDPList = zx_NEW_sp_IDPList(cf->ctx, &ar->Scoping->gg);
       /* *** Add IDPEntry and GetComplete */
     }
 #endif
@@ -82,15 +82,15 @@ struct zx_sp_ArtifactResolve_s* zxid_mk_art_deref(zxid_conf* cf, zxid_entity* id
   X509* sign_cert;
   RSA*  sign_pkey;
   struct zxsig_ref refs;
-  struct zx_sp_ArtifactResolve_s* ar = zx_NEW_sp_ArtifactResolve(cf->ctx);
+  struct zx_sp_ArtifactResolve_s* ar = zx_NEW_sp_ArtifactResolve(cf->ctx,0);
   ar->Issuer = zxid_my_issuer(cf);
-  ar->ID = zxid_mk_id(cf, "R", ZXID_ID_BITS);
-  ar->Version = zx_ref_str(cf->ctx, SAML2_VERSION);
-  ar->IssueInstant = zxid_date_time(cf, time(0));
-  ar->Artifact = zx_ref_simple_elem(cf->ctx, artifact);
+  ar->ID = zxid_mk_id_attr(cf, zx_ID_ATTR, "R", ZXID_ID_BITS);
+  ar->Version = zx_ref_attr(cf->ctx, zx_Version_ATTR, SAML2_VERSION);
+  ar->IssueInstant = zxid_date_time_attr(cf, zx_IssueInstant_ATTR, time(0));
+  ar->Artifact = zx_ref_simple_elem(cf->ctx, &ar->gg, zx_sp_Artifact_ELEM, artifact);
   if (cf->sso_soap_sign) {
     ZERO(&refs, sizeof(refs));
-    refs.id = ar->ID;
+    refs.id = &ar->ID->g;
     refs.canon = zx_EASY_ENC_SO_sp_ArtifactResolve(cf->ctx, ar);
     if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert art deref"))
       ar->Signature = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey);
@@ -104,14 +104,14 @@ struct zx_sp_ArtifactResolve_s* zxid_mk_art_deref(zxid_conf* cf, zxid_entity* id
 /* Called by:  zxid_OK */
 struct zx_sp_Status_s* zxid_mk_Status(zxid_conf* cf, char* sc1, char* sc2, char* msg)
 {
-  struct zx_sp_Status_s* st = zx_NEW_sp_Status(cf->ctx);
+  struct zx_sp_Status_s* st = zx_NEW_sp_Status(cf->ctx,0);
   if (msg)
-    st->StatusMessage = zx_ref_simple_elem(cf->ctx, msg);
-  st->StatusCode = zx_NEW_sp_StatusCode(cf->ctx);
-  st->StatusCode->Value = zx_ref_str(cf->ctx, sc1);
+    st->StatusMessage = zx_ref_simple_elem(cf->ctx, &st->gg, zx_sp_StatusMessage_ELEM, msg);
+  st->StatusCode = zx_NEW_sp_StatusCode(cf->ctx, &st->gg);
+  st->StatusCode->Value = zx_ref_attr(cf->ctx, zx_Value_ATTR, sc1);
   if (sc2) {
-    st->StatusCode->StatusCode = zx_NEW_sp_StatusCode(cf->ctx);
-    st->StatusCode->StatusCode->Value = zx_ref_str(cf->ctx, sc2);
+    st->StatusCode->StatusCode = zx_NEW_sp_StatusCode(cf->ctx, &st->StatusCode->gg);
+    st->StatusCode->StatusCode->Value = zx_ref_attr(cf->ctx, zx_Value_ATTR, sc2);
   }
   return st;
 }
@@ -130,7 +130,7 @@ struct zx_sp_Status_s* zxid_OK(zxid_conf* cf)
 /* Called by:  zxid_mk_logout, zxid_mk_mni, zxid_mk_subj */
 struct zx_sa_EncryptedID_s* zxid_mk_enc_id(zxid_conf* cf, zxid_nid* nid, zxid_entity* meta)
 {
-  struct zx_sa_EncryptedID_s* encid = zx_NEW_sa_EncryptedID(cf->ctx);
+  struct zx_sa_EncryptedID_s* encid = zx_NEW_sa_EncryptedID(cf->ctx,0);
   struct zx_str* ss = zx_EASY_ENC_SO_sa_NameID(cf->ctx, nid);
   if (cf->enckey_opt & 0x20) {
     /* Nested EncryptedKey approach (Shibboleth early 2010) */
@@ -149,7 +149,7 @@ struct zx_sa_EncryptedID_s* zxid_mk_enc_id(zxid_conf* cf, zxid_nid* nid, zxid_en
 /* Called by:  zxid_add_fed_tok_to_epr, zxid_idp_sso x4 */
 struct zx_sa_EncryptedAssertion_s* zxid_mk_enc_a7n(zxid_conf* cf, zxid_a7n* a7n, zxid_entity* meta)
 {
-  struct zx_sa_EncryptedAssertion_s* enc_a7n = zx_NEW_sa_EncryptedAssertion(cf->ctx);
+  struct zx_sa_EncryptedAssertion_s* enc_a7n = zx_NEW_sa_EncryptedAssertion(cf->ctx,0);
   struct zx_str* ss = zx_EASY_ENC_SO_sa_Assertion(cf->ctx, a7n);
   if (cf->enckey_opt & 0x20) {
     /* Nested EncryptedKey approach (Shibboleth early 2010) */
@@ -167,11 +167,11 @@ struct zx_sa_EncryptedAssertion_s* zxid_mk_enc_a7n(zxid_conf* cf, zxid_a7n* a7n,
 /* Called by:  test_ibm_cert_problem_enc_dec, zxid_sp_slo_redir, zxid_sp_slo_soap */
 struct zx_sp_LogoutRequest_s* zxid_mk_logout(zxid_conf* cf, zxid_nid* nid, struct zx_str* ses_ix, zxid_entity* idp_meta)
 {
-  struct zx_sp_LogoutRequest_s* r = zx_NEW_sp_LogoutRequest(cf->ctx);
+  struct zx_sp_LogoutRequest_s* r = zx_NEW_sp_LogoutRequest(cf->ctx,0);
   r->Issuer = zxid_my_issuer(cf);
-  r->ID = zxid_mk_id(cf, "L", ZXID_ID_BITS);
-  r->Version = zx_ref_str(cf->ctx, SAML2_VERSION);
-  r->IssueInstant = zxid_date_time(cf, time(0));
+  r->ID = zxid_mk_id_attr(cf, zx_ID_ATTR, "L", ZXID_ID_BITS);
+  r->Version = zx_ref_attr(cf->ctx, zx_Version_ATTR, SAML2_VERSION);
+  r->IssueInstant = zxid_date_time_attr(cf, zx_IssueInstant_ATTR, time(0));
 
   D("nameid_enc(%d) idp_meta(%p) enc_cert(%p)", cf->nameid_enc, idp_meta, idp_meta->enc_cert);
   if (cf->nameid_enc && idp_meta)
@@ -179,7 +179,7 @@ struct zx_sp_LogoutRequest_s* zxid_mk_logout(zxid_conf* cf, zxid_nid* nid, struc
   else
     r->NameID = nid;
   if (ses_ix)
-    r->SessionIndex = zx_new_simple_elem(cf->ctx, ses_ix);
+    r->SessionIndex = zx_new_simple_elem(cf->ctx, &r->gg, zx_sp_SessionIndex_ELEM, ses_ix);
   return r;
 }
 
@@ -188,13 +188,13 @@ struct zx_sp_LogoutRequest_s* zxid_mk_logout(zxid_conf* cf, zxid_nid* nid, struc
 /* Called by:  zxid_idp_soap_dispatch, zxid_slo_resp_redir, zxid_sp_soap_dispatch */
 struct zx_sp_LogoutResponse_s* zxid_mk_logout_resp(zxid_conf* cf, struct zx_sp_Status_s* st, struct zx_str* req_id)
 {
-  struct zx_sp_LogoutResponse_s* r = zx_NEW_sp_LogoutResponse(cf->ctx);
+  struct zx_sp_LogoutResponse_s* r = zx_NEW_sp_LogoutResponse(cf->ctx,0);
   r->Issuer = zxid_my_issuer(cf);
-  r->ID = zxid_mk_id(cf, "r", ZXID_ID_BITS);
-  r->Version = zx_ref_str(cf->ctx, SAML2_VERSION);
-  r->IssueInstant = zxid_date_time(cf, time(0));
+  r->ID = zxid_mk_id_attr(cf, zx_ID_ATTR, "r", ZXID_ID_BITS);
+  r->Version = zx_ref_attr(cf->ctx, zx_Version_ATTR, SAML2_VERSION);
+  r->IssueInstant = zxid_date_time_attr(cf, zx_IssueInstant_ATTR, time(0));
   if (req_id)
-    r->InResponseTo = req_id;
+    r->InResponseTo = zx_ref_len_attr(cf->ctx, zx_InResponseTo_ATTR, req_id->len, req_id->s);
   r->Status = st;
   return r;
 }
@@ -207,17 +207,17 @@ struct zx_sp_ManageNameIDRequest_s* zxid_mk_mni(zxid_conf* cf, zxid_nid* nid, st
 {
   struct zx_str* ss;
   struct zx_xenc_EncryptedKey_s* ek;
-  struct zx_sp_ManageNameIDRequest_s* r = zx_NEW_sp_ManageNameIDRequest(cf->ctx);
+  struct zx_sp_ManageNameIDRequest_s* r = zx_NEW_sp_ManageNameIDRequest(cf->ctx,0);
   r->Issuer = zxid_my_issuer(cf);
-  r->ID = zxid_mk_id(cf, "R", ZXID_ID_BITS);
-  r->Version = zx_ref_str(cf->ctx, SAML2_VERSION);
-  r->IssueInstant = zxid_date_time(cf, time(0));
+  r->ID = zxid_mk_id_attr(cf, zx_ID_ATTR, "R", ZXID_ID_BITS);
+  r->Version = zx_ref_attr(cf->ctx, zx_Version_ATTR, SAML2_VERSION);
+  r->IssueInstant = zxid_date_time_attr(cf, zx_IssueInstant_ATTR, time(0));
   if (cf->nameid_enc && idp_meta) {
     r->EncryptedID = zxid_mk_enc_id(cf, nid, idp_meta);
     if (new_nym && new_nym->len) {
-      struct zx_elem_s* newid = zx_new_simple_elem(cf->ctx, new_nym);
-      ss = zx_EASY_ENC_SO_simple_elem(cf->ctx, newid, "sp:NewID", sizeof("sp:NewID")-1, zx_ns_tab+zx_xmlns_ix_sp);
-      r->NewEncryptedID = zx_NEW_sp_NewEncryptedID(cf->ctx);
+      struct zx_elem_s* newid = zx_new_simple_elem(cf->ctx, 0, zx_sp_NewID_ELEM, new_nym);
+      ss = zx_EASY_ENC_SO_simple_elem(cf->ctx, newid, "sp:NewID", sizeof("sp:NewID")-1, zx_ns_tab+zx_sp_NS);
+      r->NewEncryptedID = zx_NEW_sp_NewEncryptedID(cf->ctx, &r->gg);
       if (cf->enckey_opt & 0x20) {
 	/* Nested EncryptedKey approach (Shibboleth early 2010) */
 	r->NewEncryptedID->EncryptedData = zxenc_pubkey_enc(cf, ss, 0, idp_meta->enc_cert, "43",0);
@@ -227,15 +227,15 @@ struct zx_sp_ManageNameIDRequest_s* zxid_mk_mni(zxid_conf* cf, zxid_nid* nid, st
 	r->NewEncryptedID->EncryptedKey = ek;
       }
       zx_str_free(cf->ctx, ss);
-      zx_FREE_simple_elem(cf->ctx, newid, 0);
+      zx_free_elem(cf->ctx, newid, 0);
     } else
-      r->Terminate = zx_new_simple_elem(cf->ctx, 0);
+      r->Terminate = zx_new_simple_elem(cf->ctx, &r->gg, zx_sp_Terminate_ELEM, 0);
   } else {
     r->NameID = nid;
     if (new_nym && new_nym->len)
-      r->NewID = zx_new_simple_elem(cf->ctx, new_nym);
+      r->NewID = zx_new_simple_elem(cf->ctx, &r->gg, zx_sp_NewID_ELEM, new_nym);
     else
-      r->Terminate = zx_new_simple_elem(cf->ctx, 0);
+      r->Terminate = zx_new_simple_elem(cf->ctx, &r->gg, zx_sp_Terminate_ELEM, 0);
   }
   return r;
 }
@@ -245,13 +245,13 @@ struct zx_sp_ManageNameIDRequest_s* zxid_mk_mni(zxid_conf* cf, zxid_nid* nid, st
 /* Called by:  zxid_mni_do, zxid_mni_do_ss */
 struct zx_sp_ManageNameIDResponse_s* zxid_mk_mni_resp(zxid_conf* cf, struct zx_sp_Status_s* st, struct zx_str* req_id)
 {
-  struct zx_sp_ManageNameIDResponse_s* r = zx_NEW_sp_ManageNameIDResponse(cf->ctx);
+  struct zx_sp_ManageNameIDResponse_s* r = zx_NEW_sp_ManageNameIDResponse(cf->ctx,0);
   r->Issuer = zxid_my_issuer(cf);
-  r->ID = zxid_mk_id(cf, "r", ZXID_ID_BITS);
-  r->Version = zx_ref_str(cf->ctx, SAML2_VERSION);
-  r->IssueInstant = zxid_date_time(cf, time(0));
+  r->ID = zxid_mk_id_attr(cf, zx_ID_ATTR, "r", ZXID_ID_BITS);
+  r->Version = zx_ref_attr(cf->ctx, zx_Version_ATTR, SAML2_VERSION);
+  r->IssueInstant = zxid_date_time_attr(cf, zx_IssueInstant_ATTR, time(0));
   if (req_id)
-    r->InResponseTo = req_id;
+    r->InResponseTo = zx_ref_len_attr(cf->ctx, zx_InResponseTo_ATTR, req_id->len, req_id->s);
   r->Status = st;
   return r;
 }
@@ -263,18 +263,19 @@ struct zx_sp_ManageNameIDResponse_s* zxid_mk_mni_resp(zxid_conf* cf, struct zx_s
 /* Called by:  zxid_mk_user_a7n_to_sp, zxid_xacml_az_cd1_do x2, zxid_xacml_az_do x2 */
 zxid_a7n* zxid_mk_a7n(zxid_conf* cf, struct zx_str* audience, struct zx_sa_Subject_s* subj, struct zx_sa_AuthnStatement_s* an_stmt, struct zx_sa_AttributeStatement_s* at_stmt, struct zx_xasa_XACMLAuthzDecisionStatement_s* az_stmt)
 {
-  zxid_a7n* a7n =  zx_NEW_sa_Assertion(cf->ctx);
-  a7n->Version = zx_dup_str(cf->ctx, SAML2_VERSION);
-  a7n->ID = zxid_mk_id(cf, "A", ZXID_ID_BITS);
+  zxid_a7n* a7n =  zx_NEW_sa_Assertion(cf->ctx,0);
+  a7n->Version = zx_dup_attr(cf->ctx, zx_Version_ATTR, SAML2_VERSION);
+  a7n->ID = zxid_mk_id_attr(cf, zx_ID_ATTR, "A", ZXID_ID_BITS);
   a7n->Issuer = zxid_my_issuer(cf);
-  a7n->IssueInstant = zxid_date_time(cf, time(0));
+  a7n->IssueInstant = zxid_date_time_attr(cf, zx_IssueInstant_ATTR, time(0));
   a7n->Subject = subj;
-  a7n->Conditions = zx_NEW_sa_Conditions(cf->ctx);
-  a7n->Conditions->NotBefore = zxid_date_time(cf, time(0));
-  a7n->Conditions->NotOnOrAfter = zxid_date_time(cf, time(0) + cf->a7nttl);
+  a7n->Conditions = zx_NEW_sa_Conditions(cf->ctx, &a7n->gg);
+  a7n->Conditions->NotBefore = zxid_date_time_attr(cf, zx_NotBefore_ATTR, time(0));
+  a7n->Conditions->NotOnOrAfter = zxid_date_time_attr(cf, zx_NotOnOrAfter_ATTR, time(0) + cf->a7nttl);
   if (audience) {
-    a7n->Conditions->AudienceRestriction = zx_NEW_sa_AudienceRestriction(cf->ctx);
-    a7n->Conditions->AudienceRestriction->Audience = zx_new_simple_elem(cf->ctx, audience);
+    a7n->Conditions->AudienceRestriction = zx_NEW_sa_AudienceRestriction(cf->ctx, &a7n->Conditions->gg);
+    a7n->Conditions->AudienceRestriction->Audience
+      = zx_new_simple_elem(cf->ctx, &a7n->Conditions->AudienceRestriction->gg, zx_sa_Audience_ELEM, audience);
   }
   a7n->AuthnStatement = an_stmt;
   a7n->AttributeStatement = at_stmt;
@@ -287,16 +288,16 @@ zxid_a7n* zxid_mk_a7n(zxid_conf* cf, struct zx_str* audience, struct zx_sa_Subje
 /* Called by:  zxid_mk_user_a7n_to_sp, zxid_xacml_az_cd1_do, zxid_xacml_az_do */
 struct zx_sa_Subject_s* zxid_mk_subj(zxid_conf* cf, zxid_entity* sp_meta, zxid_nid* nid)
 {
-  struct zx_sa_Subject_s* subj = zx_NEW_sa_Subject(cf->ctx);
+  struct zx_sa_Subject_s* subj = zx_NEW_sa_Subject(cf->ctx,0);
 
 #if 0
   // , struct zx_str* affil, char* fmt
-  nid = zx_NEW_sa_NameID(cf->ctx);
+  nid = zx_NEW_sa_NameID(cf->ctx,0);
   nid->Format = zx_dup_str(cf->ctx, fmt);  /* *** implement persistent */
   nid->NameQualifier = zxid_my_entity_id(cf);
   nid->SPNameQualifier = affil;
   if (!strcmp(fmt, SAML2_TRANSIENT_NID_FMT)) {
-    nid->gg.content = zxid_mk_id(cf, "T", ZXID_ID_BITS);
+    zx_add_content(cf->ctx, nid, zxid_mk_id(cf, "T", ZXID_ID_BITS));
   } else {
     /* *** see also zxid_get_user_nameid() */
   }
@@ -322,7 +323,8 @@ struct zx_sa_AuthnStatement_s* zxid_mk_an_stmt(zxid_conf* cf, zxid_ses* ses, con
 {
   struct zx_str sesix;
   struct zx_str eid_ss;
-  struct zx_sa_AuthnStatement_s* an_stmt = zx_NEW_sa_AuthnStatement(cf->ctx);
+  struct zx_str* ss;
+  struct zx_sa_AuthnStatement_s* an_stmt = zx_NEW_sa_AuthnStatement(cf->ctx,0);
   if (ses->sesix) {
 #if 0
     an_stmt->SessionIndex = zx_dup_str(cf->ctx, ses->sesix);
@@ -332,13 +334,15 @@ struct zx_sa_AuthnStatement_s* zxid_mk_an_stmt(zxid_conf* cf, zxid_ses* ses, con
     eid_ss.s = (char*)eid;
     sesix.len = strlen(ses->sesix);
     sesix.s = ses->sesix;
-    an_stmt->SessionIndex = zxid_psobj_enc(cf, &eid_ss, "ZS", &sesix);
+    ss = zxid_psobj_enc(cf, &eid_ss, "ZS", &sesix);
+    an_stmt->SessionIndex = zx_ref_len_attr(cf->ctx, zx_SessionIndex_ATTR, ss->len, ss->s);
 #endif
   }
-  an_stmt->AuthnInstant = zxid_date_time(cf, ses->an_instant);
-  an_stmt->AuthnContext = zx_NEW_sa_AuthnContext(cf->ctx);
+  an_stmt->AuthnInstant = zxid_date_time_attr(cf, zx_AuthnInstant_ATTR, ses->an_instant);
+  an_stmt->AuthnContext = zx_NEW_sa_AuthnContext(cf->ctx, &an_stmt->gg);
   if (ses->an_ctx)
-    an_stmt->AuthnContext->AuthnContextClassRef = zx_dup_simple_elem(cf->ctx, ses->an_ctx);
+    an_stmt->AuthnContext->AuthnContextClassRef
+      = zx_dup_simple_elem(cf->ctx, &an_stmt->AuthnContext->gg, zx_sa_AuthnContextClassRef_ELEM, ses->an_ctx);
   else {
     ERR("Session(%s) lacks AuthentContextClassRef. Output AuthnStatement will not satisfy all processing rules. See configuration option ISSUE_AUTHNCTX_PW.", ses->sesix);
   }
@@ -350,11 +354,11 @@ struct zx_sa_AuthnStatement_s* zxid_mk_an_stmt(zxid_conf* cf, zxid_ses* ses, con
 /* Called by:  zxid_add_ldif_attrs, zxid_gen_boots, zxid_mk_user_a7n_to_sp */
 struct zx_sa_Attribute_s* zxid_mk_attribute(zxid_conf* cf, char* name, char* val)
 {
-  struct zx_sa_Attribute_s* r = zx_NEW_sa_Attribute(cf->ctx);
-  r->Name = zx_dup_str(cf->ctx, name);
-  r->AttributeValue = zx_NEW_sa_AttributeValue(cf->ctx);
+  struct zx_sa_Attribute_s* r = zx_NEW_sa_Attribute(cf->ctx,0);
+  r->Name = zx_dup_attr(cf->ctx, zx_Name_ATTR, name);
+  r->AttributeValue = zx_NEW_sa_AttributeValue(cf->ctx, &r->gg);
   if (val)
-    r->AttributeValue->gg.content = zx_dup_str(cf->ctx, val);
+    zx_add_content(cf->ctx, &r->AttributeValue->gg, zx_dup_str(cf->ctx, val));
   return r;
 }
 
@@ -363,11 +367,11 @@ struct zx_sa_Attribute_s* zxid_mk_attribute(zxid_conf* cf, char* name, char* val
 /* Called by:  zxid_idp_sso x4, zxid_xacml_az_cd1_do x2, zxid_xacml_az_do x2 */
 struct zx_sp_Response_s* zxid_mk_saml_resp(zxid_conf* cf)
 {
-  struct zx_sp_Response_s* r = zx_NEW_sp_Response(cf->ctx);
-  r->Version = zx_dup_str(cf->ctx, SAML2_VERSION);
-  r->ID = zxid_mk_id(cf, "R", ZXID_ID_BITS);
+  struct zx_sp_Response_s* r = zx_NEW_sp_Response(cf->ctx,0);
+  r->Version = zx_dup_attr(cf->ctx, zx_Version_ATTR, SAML2_VERSION);
+  r->ID = zxid_mk_id_attr(cf, zx_ID_ATTR, "R", ZXID_ID_BITS);
   r->Issuer = zxid_my_issuer(cf);
-  r->IssueInstant = zxid_date_time(cf, time(0));
+  r->IssueInstant = zxid_date_time_attr(cf, zx_IssueInstant_ATTR, time(0));
   r->Status = zxid_OK(cf);
   return r;
 }
@@ -377,26 +381,27 @@ struct zx_sp_Response_s* zxid_mk_saml_resp(zxid_conf* cf)
 /* Called by:  zxid_ins_xacml_az_cd1_stmt x2, zxid_ins_xacml_az_stmt x2 */
 struct zx_xac_Response_s* zxid_mk_xacml_resp(zxid_conf* cf, char* decision)
 {
-  struct zx_xac_Response_s* resp = zx_NEW_xac_Response(cf->ctx);
-  resp->Result = zx_NEW_xac_Result(cf->ctx);
-  resp->Result->Decision = zx_ref_simple_elem(cf->ctx, decision);
-  resp->Result->Status = zx_NEW_xac_Status(cf->ctx);
-  resp->Result->Status->StatusCode = zx_NEW_xac_StatusCode(cf->ctx);
+  struct zx_xac_Response_s* resp = zx_NEW_xac_Response(cf->ctx,0);
+  resp->Result = zx_NEW_xac_Result(cf->ctx, &resp->gg);
+  resp->Result->Decision = zx_ref_simple_elem(cf->ctx, &resp->Result->gg, zx_xac_Decision_ELEM, decision);
+  resp->Result->Status = zx_NEW_xac_Status(cf->ctx, &resp->Result->gg);
+  resp->Result->Status->StatusCode = zx_NEW_xac_StatusCode(cf->ctx, &resp->Result->Status->gg);
   resp->Result->Status->StatusCode->Value
-    = zx_ref_str(cf->ctx, "urn:oasis:names:tc:xacml:1.0:status:ok");
+    = zx_ref_attr(cf->ctx, zx_Value_ATTR, "urn:oasis:names:tc:xacml:1.0:status:ok");
   return resp;
 }
 
 /* Called by:  zxid_pep_az_soap x3 */
 struct zx_xac_Attribute_s* zxid_mk_xacml_simple_at(zxid_conf* cf, struct zx_xac_Attribute_s* aa, struct zx_str* atid, struct zx_str* attype, struct zx_str* atissuer, struct zx_str* atvalue)
 {
-  struct zx_xac_Attribute_s* at = zx_NEW_xac_Attribute(cf->ctx);
+  struct zx_xac_Attribute_s* at = zx_NEW_xac_Attribute(cf->ctx,0);
   ZX_NEXT(at) = (void*)aa;
-  at->AttributeId = atid;
-  at->DataType = attype;
-  at->Issuer = atissuer;
-  //at->AttributeValue = zx_NEW_xac_AttributeValue(cf->ctx);
-  at->AttributeValue = zx_new_simple_elem(cf->ctx, atvalue);
+  at->AttributeId = zx_ref_len_attr(cf->ctx, zx_AttributeId_ATTR, atid->len, atid->s);
+  at->DataType = zx_ref_len_attr(cf->ctx, zx_DataType_ATTR, attype->len, attype->s);
+  if (atissuer)
+    at->Issuer = zx_ref_len_attr(cf->ctx, zx_Issuer_ATTR, atissuer->len, atissuer->s);
+  //at->AttributeValue = zx_NEW_xac_AttributeValue(cf->ctx, &at->gg);
+  at->AttributeValue = zx_new_simple_elem(cf->ctx, &at->gg, zx_xac_AttributeValue_ELEM, atvalue);
   return at;
 }
 
@@ -404,14 +409,14 @@ struct zx_xac_Attribute_s* zxid_mk_xacml_simple_at(zxid_conf* cf, struct zx_xac_
 
 struct zx_xac_Request_s* zxid_mk_xac_az(zxid_conf* cf, struct zx_xac_Attribute_s* subj, struct zx_xac_Attribute_s* rsrc, struct zx_xac_Attribute_s* act, struct zx_xac_Attribute_s* env)
 {
-  struct zx_xac_Request_s* r = zx_NEW_xac_Request(cf->ctx);
-  r->Subject  = zx_NEW_xac_Subject(cf->ctx);
+  struct zx_xac_Request_s* r = zx_NEW_xac_Request(cf->ctx,0);
+  r->Subject  = zx_NEW_xac_Subject(cf->ctx, &r->gg);
   r->Subject->Attribute = subj;
-  r->Resource = zx_NEW_xac_Resource(cf->ctx);
+  r->Resource = zx_NEW_xac_Resource(cf->ctx, &r->gg);
   r->Resource->Attribute = rsrc;
-  r->Action   = zx_NEW_xac_Action(cf->ctx);
+  r->Action   = zx_NEW_xac_Action(cf->ctx, &r->gg);
   r->Action->Attribute = act;
-  r->Environment = zx_NEW_xac_Environment(cf->ctx);
+  r->Environment = zx_NEW_xac_Environment(cf->ctx, &r->gg);
   r->Environment->Attribute = env;
   return r;
 }
@@ -421,11 +426,11 @@ struct zx_xac_Request_s* zxid_mk_xac_az(zxid_conf* cf, struct zx_xac_Attribute_s
 /* Called by:  zxid_pep_az_soap */
 struct zx_xasp_XACMLAuthzDecisionQuery_s* zxid_mk_az(zxid_conf* cf, struct zx_xac_Attribute_s* subj, struct zx_xac_Attribute_s* rsrc, struct zx_xac_Attribute_s* act, struct zx_xac_Attribute_s* env)
 {
-  struct zx_xasp_XACMLAuthzDecisionQuery_s* r = zx_NEW_xasp_XACMLAuthzDecisionQuery(cf->ctx);
+  struct zx_xasp_XACMLAuthzDecisionQuery_s* r = zx_NEW_xasp_XACMLAuthzDecisionQuery(cf->ctx,0);
   r->Issuer = zxid_my_issuer(cf);
-  r->ID = zxid_mk_id(cf, "R", ZXID_ID_BITS);
-  r->Version = zx_ref_str(cf->ctx, SAML2_VERSION);
-  r->IssueInstant = zxid_date_time(cf, time(0));
+  r->ID = zxid_mk_id_attr(cf, zx_ID_ATTR, "R", ZXID_ID_BITS);
+  r->Version = zx_ref_attr(cf->ctx, zx_Version_ATTR, SAML2_VERSION);
+  r->IssueInstant = zxid_date_time_attr(cf, zx_IssueInstant_ATTR, time(0));
   r->Request = zxid_mk_xac_az(cf, subj, rsrc, act, env);
   return r;
 }
@@ -435,11 +440,11 @@ struct zx_xasp_XACMLAuthzDecisionQuery_s* zxid_mk_az(zxid_conf* cf, struct zx_xa
 /* Called by:  zxid_pep_az_soap x2 */
 struct zx_xaspcd1_XACMLAuthzDecisionQuery_s* zxid_mk_az_cd1(zxid_conf* cf, struct zx_xac_Attribute_s* subj, struct zx_xac_Attribute_s* rsrc, struct zx_xac_Attribute_s* act, struct zx_xac_Attribute_s* env)
 {
-  struct zx_xaspcd1_XACMLAuthzDecisionQuery_s* r = zx_NEW_xaspcd1_XACMLAuthzDecisionQuery(cf->ctx);
+  struct zx_xaspcd1_XACMLAuthzDecisionQuery_s* r = zx_NEW_xaspcd1_XACMLAuthzDecisionQuery(cf->ctx,0);
   r->Issuer = zxid_my_issuer(cf);
-  r->ID = zxid_mk_id(cf, "R", ZXID_ID_BITS);
-  r->Version = zx_ref_str(cf->ctx, SAML2_VERSION);
-  r->IssueInstant = zxid_date_time(cf, time(0));
+  r->ID = zxid_mk_id_attr(cf, zx_ID_ATTR, "R", ZXID_ID_BITS);
+  r->Version = zx_ref_attr(cf->ctx, zx_Version_ATTR, SAML2_VERSION);
+  r->IssueInstant = zxid_date_time_attr(cf, zx_IssueInstant_ATTR, time(0));
   r->Request = zxid_mk_xac_az(cf, subj, rsrc, act, env);
   return r;
 }
