@@ -24,6 +24,9 @@
 #include "c/zx-data.h"
 #include "c/zx-e-data.h"
 
+#define zx_e_actor_ATTR (zx_e_NS|zx_actor_ATTR)
+#define zx_e_mustUnderstand_ATTR (zx_e_NS|zx_mustUnderstand_ATTR)
+
 #if 0
 #define XS_STRING "xs:string"
 #else
@@ -168,14 +171,14 @@ static struct zx_sp_Response_s* zxid_az_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_
   struct zx_str* ss;
   struct zx_sp_Response_s* resp;
 
-  hdr = zx_NEW_e_Header(cf->ctx);
+  hdr = zx_NEW_e_Header(cf->ctx,0);
 #if 0
-  hdr->Action = zx_NEW_a_Action(cf->ctx);
-  //hdr->Action->gg.content = zx_dup_str(cf->ctx, "urn:oasis:names:tc:xacml:2.0:profile:saml2.0:v2:schema:protocol:cd-01");
-  hdr->Action->gg.content = zx_dup_str(cf->ctx, "urn:oasis:xacml:2.0:saml:protocol:schema:os");
+  hdr->Action = zx_NEW_a_Action(cf->ctx, &hdr->gg);
+  //zx_add_content(cf->ctx, &hdr->Action->gg, zx_dup_str(cf->ctx, "urn:oasis:names:tc:xacml:2.0:profile:saml2.0:v2:schema:protocol:cd-01"));
+  zx_add_content(cf->ctx, &hdr->Action->gg, zx_dup_str(cf->ctx, "urn:oasis:xacml:2.0:saml:protocol:schema:os"));
 
-  //hdr->Action->gg.content = zx_dup_str(cf->ctx, "SAML2XACMLAuthzRequest");
-  //hdr->Action->gg.content = zx_dup_str(cf->ctx, "http://ws.apache.org/axis2/TestPolicyPortType/authRequestRequest");
+  //zx_add_content(cf->ctx, &hdr->Action->gg, zx_dup_str(cf->ctx, "SAML2XACMLAuthzRequest"));
+  //zx_add_content(cf->ctx, &hdr->Action->gg, zx_dup_str(cf->ctx, "http://ws.apache.org/axis2/TestPolicyPortType/authRequestRequest"));
   hdr->Action->actor = zx_ref_str(cf->ctx, SOAP_ACTOR_NEXT);
   hdr->Action->mustUnderstand = zx_ref_str(cf->ctx, ZXID_TRUE);
 #endif
@@ -183,23 +186,23 @@ static struct zx_sp_Response_s* zxid_az_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_
   /* Add our own token so PDP can do whatever PEP can (they are considered to be
    * part of the same entity). This is TAS3 specific hack. */
 
-  sec = hdr->Security = zx_NEW_wsse_Security(cf->ctx);
-  sec->actor = zx_ref_str(cf->ctx, SOAP_ACTOR_NEXT);
-  sec->mustUnderstand = zx_ref_str(cf->ctx, ZXID_TRUE);
-  sec->Timestamp = zx_NEW_wsu_Timestamp(cf->ctx);
-  sec->Timestamp->Created = zx_NEW_wsu_Created(cf->ctx);
+  sec = hdr->Security = zx_NEW_wsse_Security(cf->ctx, &hdr->gg);
+  sec->actor = zx_ref_attr(cf->ctx, zx_e_actor_ATTR, SOAP_ACTOR_NEXT);
+  sec->mustUnderstand = zx_ref_attr(cf->ctx, zx_e_mustUnderstand_ATTR, ZXID_TRUE);
+  sec->Timestamp = zx_NEW_wsu_Timestamp(cf->ctx, &sec->gg);
+  sec->Timestamp->Created = zx_NEW_wsu_Created(cf->ctx, &sec->Timestamp->gg);
   sec->Assertion = ses->tgta7n;
   D("tgta7n=%p", ses->tgta7n);
 
   /* Prepare request according to the version */
 
-  body = zx_NEW_e_Body(cf->ctx);
+  body = zx_NEW_e_Body(cf->ctx,0);
   if (!strcmp(cf->xasp_vers, "xac-soap")) {
     body->xac_Request = zxid_mk_xac_az(cf, subj, rsrc, act, env);
 #if 0
     /* *** xac:Response does not have signature field */
     if (cf->sso_soap_sign) {
-      ZERO(refs, sizeof(refs));
+      ZERO(&refs, sizeof(refs));
       refs.id = body->xac_Request->ID;
       refs.canon = zx_EASY_ENC_SO_xac_Request(cf->ctx, body->xac_Request);
       if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert az xac-soap"))
@@ -212,7 +215,7 @@ static struct zx_sp_Response_s* zxid_az_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_
     body->xaspcd1_XACMLAuthzDecisionQuery = zxid_mk_az_cd1(cf, subj, rsrc, act, env);
     if (cf->sso_soap_sign) {
       ZERO(&refs, sizeof(refs));
-      refs.id = body->xaspcd1_XACMLAuthzDecisionQuery->ID;
+      refs.id = &body->xaspcd1_XACMLAuthzDecisionQuery->ID->g;
       refs.canon = zx_EASY_ENC_SO_xaspcd1_XACMLAuthzDecisionQuery(cf->ctx, body->xaspcd1_XACMLAuthzDecisionQuery);
       if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert az cd1"))
 	body->xaspcd1_XACMLAuthzDecisionQuery->Signature
@@ -223,7 +226,7 @@ static struct zx_sp_Response_s* zxid_az_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_
     body->XACMLAuthzDecisionQuery = zxid_mk_az(cf, subj, rsrc, act, env);
     if (cf->sso_soap_sign) {
       ZERO(&refs, sizeof(refs));
-      refs.id = body->XACMLAuthzDecisionQuery->ID;
+      refs.id = &body->XACMLAuthzDecisionQuery->ID->g;
       refs.canon = zx_EASY_ENC_SO_xasp_XACMLAuthzDecisionQuery(cf->ctx, body->XACMLAuthzDecisionQuery);
       if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert az"))
 	body->XACMLAuthzDecisionQuery->Signature
@@ -302,7 +305,7 @@ char* zxid_pep_az_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, const
   struct zx_elem_s* decision;
 
   if (cf->log_level>0)
-    zxlog(cf, 0, 0, 0, 0, 0, 0, ses&&ses->nameid?ses->nameid->gg.content:0, "N", "W", "AZSOAP", ses?ses->sid:0, " ");
+    zxlog(cf, 0, 0, 0, 0, 0, 0, ses?ZX_GET_CONTENT(ses->nameid):0, "N", "W", "AZSOAP", ses?ses->sid:0, " ");
   
   if (!pdp_url || !*pdp_url) {
     ERR("No PDP_URL or PDP_CALL_URL set. Deny. %p", pdp_url);
@@ -318,7 +321,7 @@ char* zxid_pep_az_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, const
   if (az_stmt && az_stmt->Response && az_stmt->Response->Result) {
     decision = az_stmt->Response->Result->Decision;
     if (ZX_CONTENT_EQ_CONST(decision, "Permit")) {
-      ss = zx_EASY_ENC_WO_xac_Response(cf->ctx, az_stmt->Response);
+      ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &az_stmt->Response->gg);
       if (!ss || !ss->len)
 	return 0;
       res = ss->s;
@@ -331,7 +334,7 @@ char* zxid_pep_az_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, const
   if (az_stmt_cd1 && az_stmt_cd1->Response && az_stmt_cd1->Response->Result) {
     decision = az_stmt_cd1->Response->Result->Decision;
     if (ZX_CONTENT_EQ_CONST(decision, "Permit")) {
-      ss = zx_EASY_ENC_WO_xac_Response(cf->ctx, az_stmt_cd1->Response);
+      ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &az_stmt_cd1->Response->gg);
       if (!ss || !ss->len)
 	return 0;
       res = ss->s;
@@ -344,7 +347,7 @@ char* zxid_pep_az_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, const
   if (stmt && stmt->Response && stmt->Response->Result) {  /* Response here is xac:Response */
     decision = stmt->Response->Result->Decision;
     if (ZX_CONTENT_EQ_CONST(decision, "Permit")) {
-      ss = zx_EASY_ENC_WO_xac_Response(cf->ctx, stmt->Response);
+      ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &stmt->Response->gg);
       if (!ss || !ss->len)
 	return 0;
       res = ss->s;
@@ -399,7 +402,7 @@ char* zxid_pep_az_base_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses,
   struct zx_xasacd1_XACMLAuthzDecisionStatement_s* az_stmt_cd1;
 
   if (cf->log_level>0)
-    zxlog(cf, 0, 0, 0, 0, 0, 0, ses&&ses->nameid?ses->nameid->gg.content:0, "N", "W", "AZSOAP", ses?ses->sid:0, " ");
+    zxlog(cf, 0, 0, 0, 0, 0, 0, ses?ZX_GET_CONTENT(ses->nameid):0, "N", "W", "AZSOAP", ses?ses->sid:0, " ");
   
   if (!pdp_url || !*pdp_url) {
     ERR("No PDP_URL or PDP_CALL_URL set. Deny. %p", pdp_url);
@@ -413,7 +416,7 @@ char* zxid_pep_az_base_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses,
 
   az_stmt = resp->Assertion->XACMLAuthzDecisionStatement;
   if (az_stmt && az_stmt->Response) {
-    ss = zx_EASY_ENC_WO_xac_Response(cf->ctx, az_stmt->Response);
+    ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &az_stmt->Response->gg);
     if (!ss || !ss->len)
       return 0;
     res = ss->s;
@@ -423,7 +426,7 @@ char* zxid_pep_az_base_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses,
   }
   az_stmt_cd1 = resp->Assertion->xasacd1_XACMLAuthzDecisionStatement;
   if (az_stmt_cd1 && az_stmt_cd1->Response) {
-    ss = zx_EASY_ENC_WO_xac_Response(cf->ctx, az_stmt_cd1->Response);
+    ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &az_stmt_cd1->Response->gg);
     if (!ss || !ss->len)
       return 0;
     res = ss->s;
@@ -433,7 +436,7 @@ char* zxid_pep_az_base_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses,
   }
   stmt = resp->Assertion->Statement;
   if (stmt && stmt->Response) {  /* Response here is xac:Response */
-    ss = zx_EASY_ENC_WO_xac_Response(cf->ctx, stmt->Response);
+    ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &stmt->Response->gg);
     if (!ss || !ss->len)
       return 0;
     res = ss->s;
