@@ -166,9 +166,9 @@ struct zx_str* zxid_start_sso_url(zxid_conf* cf, zxid_cgi* cgi)
       return 0;
     }
     for (sso_svc = idp_meta->ed->IDPSSODescriptor->SingleSignOnService;
-	 sso_svc;
+	 sso_svc && sso_svc->gg.g.tok == zx_md_SingleSignOnService_ELEM;
 	 sso_svc = (struct zx_md_SingleSignOnService_s*)sso_svc->gg.g.n)
-      if (sso_svc->Binding && !memcmp(SAML2_REDIR, sso_svc->Binding->s, sso_svc->Binding->len))
+      if (sso_svc->Binding && !memcmp(SAML2_REDIR, sso_svc->Binding->g.s, sso_svc->Binding->g.len))
 	break;
     if (!sso_svc) {
       ERR("IdP Entity(%s) does not have any IdP SSO Service with " SAML2_REDIR " binding (metadata problem)", cgi->eid);
@@ -190,7 +190,7 @@ struct zx_str* zxid_start_sso_url(zxid_conf* cf, zxid_cgi* cgi)
   }
   if (cf->log_level>0)
     zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "W", "ANREDIR", cgi->eid, 0);
-  ars = zxid_saml2_redir_url(cf, sso_svc->Location, ars, cgi->rs);
+  ars = zxid_saml2_redir_url(cf, &sso_svc->Location->g, ars, cgi->rs);
   D_DEDENT("start_sso: ");
   return ars;
 }
@@ -245,7 +245,7 @@ int zxid_sp_deref_art(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses)
   D_INDENT("deref: ");
   len = strlen(cgi->saml_art);
   if (cf->log_level > 0)
-    zxlog(cf, 0, 0, 0, 0, 0, 0, ses->nameid?ses->nameid->gg.content:0, "N", "W", "ART", cgi->saml_art, 0);
+    zxlog(cf, 0, 0, 0, 0, 0, 0, ZX_GET_CONTENT(ses->nameid), "N", "W", "ART", cgi->saml_art, 0);
   if (len-7 >= sizeof(buf)*4/3) {
     ERR("SAMLart(%s), %d chars, too long. Max(%d) chars.", cgi->saml_art, len, sizeof(buf)*4/3+6);
     zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "C", "ERR", cgi->saml_art, "Artifact too long");
@@ -289,10 +289,10 @@ int zxid_sp_deref_art(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses)
       return 0;
     }
     for (ar_svc = idp_meta->ed->IDPSSODescriptor->ArtifactResolutionService;
-	 ar_svc;
+	 ar_svc && ar_svc->gg.g.tok == zx_md_ArtifactResolutionService_ELEM;
 	 ar_svc = (struct zx_md_ArtifactResolutionService_s*)ar_svc->gg.g.n)
-      if (ar_svc->Binding  && !memcmp(SAML2_SOAP, ar_svc->Binding->s, ar_svc->Binding->len)
-	  && ar_svc->index && !memcmp(end_pt_ix, ar_svc->index->s, ar_svc->index->len)
+      if (ar_svc->Binding  && !memcmp(SAML2_SOAP, ar_svc->Binding->g.s, ar_svc->Binding->g.len)
+	  && ar_svc->index && !memcmp(end_pt_ix, ar_svc->index->g.s, ar_svc->index->g.len)
 	  && ar_svc->Location)
 	break;
     if (!ar_svc) {
@@ -302,9 +302,9 @@ int zxid_sp_deref_art(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses)
       return 0;
     }
     
-    body = zx_NEW_e_Body(cf->ctx);
+    body = zx_NEW_e_Body(cf->ctx,0);
     body->ArtifactResolve = zxid_mk_art_deref(cf, idp_meta, cgi->saml_art);
-    r = zxid_soap_call_body(cf, ar_svc->Location, body);
+    r = zxid_soap_call_body(cf, &ar_svc->Location->g, body);
     len =  zxid_sp_soap_dispatch(cf, cgi, ses, r);
     D_DEDENT("deref: ");
     return len;
@@ -404,12 +404,17 @@ int zxid_validate_cond(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n* a7
   struct timeval tsbuf;
   struct zx_sa_AudienceRestriction_s* audr;
   struct zx_elem_s* aud;
+  struct zx_str* ss;
   int secs;
 
   if (!a7n || !a7n->Conditions) {
     INFO("Assertion does not have Conditions. %p", a7n);
     return ZXSIG_OK;
   }
+  if (!myentid || !myentid->len) {
+    ERR("My entity ID missing %p", myentid);
+    return ZXSIG_OK;
+  }
 
   if (!ourts) {
     GETTIMEOFDAY(&tsbuf, 0);
@@ -417,13 +422,19 @@ int zxid_validate_cond(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n* a7
   }
 
   if (a7n->Conditions->AudienceRestriction) {
-    for (audr = a7n->Conditions->AudienceRestriction; audr; audr = (struct zx_sa_AudienceRestriction_s*)audr->gg.g.n)
-      for (aud = audr->Audience; aud; aud = (struct zx_elem_s*)aud->g.n)
-	if (aud->content->len == myentid->len
-	    && !memcmp(aud->content->s, myentid->s, aud->content->len)) {
+    for (audr = a7n->Conditions->AudienceRestriction;
+	 audr && audr->gg.g.tok == zx_sa_AudienceRestriction_ELEM;
+	 audr = (struct zx_sa_AudienceRestriction_s*)audr->gg.g.n) {
+      for (aud = audr->Audience;
+	   aud && aud->g.tok == zx_sa_Audience_ELEM;
+	   aud = (struct zx_elem_s*)aud->g.n) {
+	ss = ZX_GET_CONTENT(aud);
+	if (ss?ss->len:0 == myentid->len && !memcmp(ss->s, myentid->s, ss->len)) {
 	  D("Found audience. %d", 0);
 	  goto found_audience;
 	}
+      }
+    }
     if (cgi) {
       cgi->sigval = "V";
       cgi->sigmsg = "This SP not included in the Assertion Audience.";
@@ -443,8 +454,8 @@ int zxid_validate_cond(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n* a7
   }
  found_audience:
   
-  if (a7n->Conditions->NotOnOrAfter && a7n->Conditions->NotOnOrAfter->len > 18) {
-    secs = zx_date_time_to_secs(a7n->Conditions->NotOnOrAfter->s);
+  if (a7n->Conditions->NotOnOrAfter && a7n->Conditions->NotOnOrAfter->g.len > 18) {
+    secs = zx_date_time_to_secs(a7n->Conditions->NotOnOrAfter->g.s);
     if (secs <= ourts->tv_sec) {
       if (secs + cf->after_slop <= ourts->tv_sec) {
 	ERR("NotOnOrAfter rejected with slop of %d. Time to expiry %ld secs. Our gettimeofday: %ld secs, remote: %d secs", cf->after_slop, secs - ourts->tv_sec, ourts->tv_sec, secs);
@@ -469,8 +480,8 @@ int zxid_validate_cond(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n* a7
     INFO("Assertion does not have NotOnOrAfter. %d", 0);
   }
   
-  if (a7n->Conditions->NotBefore && a7n->Conditions->NotBefore->len > 18) {
-    secs = zx_date_time_to_secs(a7n->Conditions->NotBefore->s);
+  if (a7n->Conditions->NotBefore && a7n->Conditions->NotBefore->g.len > 18) {
+    secs = zx_date_time_to_secs(a7n->Conditions->NotBefore->g.s);
     if (secs > ourts->tv_sec) {
       if (secs - cf->before_slop > ourts->tv_sec) {
 	ERR("NotBefore rejected with slop of %d. Time to validity %ld secs. Our gettimeofday: %ld secs, remote: %d secs", cf->before_slop, secs - ourts->tv_sec, ourts->tv_sec, secs);
@@ -497,7 +508,7 @@ int zxid_validate_cond(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n* a7
   return ZXSIG_OK;
 }
 
-struct zx_str unknown_str = {{0,0,0,0,0}, 1, "??"};  /* Static string used as dummy value. */
+struct zx_str unknown_str = {0,0,1,"??"};  /* Static string used as dummy value. */
 
 /*(i) zxid_sp_sso_finalize() gets called irrespective of binding (POST, Artifact)
  * and validates the SSO a7n, including the authentication statement.
@@ -533,38 +544,31 @@ int zxid_sp_sso_finalize(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n*
     ERR("SSO failed: no assertion supplied, or assertion didn't contain AuthnStatement. %p", a7n);
     goto erro;
   }
-  if (!a7n->IssueInstant || !a7n->IssueInstant->len || !a7n->IssueInstant->s || !a7n->IssueInstant->s[0]) {
+  if (!a7n->IssueInstant || !a7n->IssueInstant->g.len || !a7n->IssueInstant->g.s || !a7n->IssueInstant->g.s[0]) {
     ERR("SSO failed: assertion does not have IssueInstant or it is empty. %p", a7n->IssueInstant);
     goto erro;
   }
-  srcts.tv_sec = zx_date_time_to_secs(a7n->IssueInstant->s);
-  if (!a7n->Issuer || !a7n->Issuer->gg.content) {
+  srcts.tv_sec = zx_date_time_to_secs(a7n->IssueInstant->g.s);
+  if (!(issuer = ZX_GET_CONTENT(a7n->Issuer))) {
     ERR("SSO failed: assertion does not have Issuer. %p", a7n->Issuer);
     goto erro;
   }
   
   /* See zxid_wsp_validate() for similar code. *** consider factoring out commonality */
   
-  issuer = a7n->Issuer->gg.content;
-  if (!issuer || !issuer->len || !issuer->s[0]) {
-    ERR("SSO failed: Issuer of the assertion is empty. %d", issuer->len);
-    goto erro;
-  }
-
   if (!a7n->Subject) {
     ERR("SSO failed: assertion does not have Subject. %p", a7n);
     goto erro;
   }
 
   ses->nameid = zxid_decrypt_nameid(cf, a7n->Subject->NameID, a7n->Subject->EncryptedID);
-  if (!ses->nameid || !ses->nameid->gg.content) {
+  if (!(subj = ZX_GET_CONTENT(ses->nameid))) {
     ERR("SSO failed: assertion does not have Subject->NameID. %p", ses->nameid);
     goto erro;
   }
   
-  subj = ses->nameid->gg.content;
   ses->nid = zx_str_to_c(cf->ctx, subj);
-  if (ses->nameid->Format && !memcmp(ses->nameid->Format->s, SAML2_TRANSIENT_NID_FMT, ses->nameid->Format->len)) {
+  if (ses->nameid->Format && !memcmp(ses->nameid->Format->g.s, SAML2_TRANSIENT_NID_FMT, ses->nameid->Format->g.len)) {
     ses->nidfmt = 0;
   } else {
     ses->nidfmt = 1;  /* anything nontransient may be a federation */
@@ -577,7 +581,7 @@ int zxid_sp_sso_finalize(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n*
   ses->tgtfmt = ses->nidfmt;
 
   if (a7n->AuthnStatement->SessionIndex)
-    ses->sesix = zx_str_to_c(cf->ctx, a7n->AuthnStatement->SessionIndex);
+    ses->sesix = zx_str_to_c(cf->ctx, &a7n->AuthnStatement->SessionIndex->g);
   
   D("SSOA7N received. NID(%s) FMT(%d) SESIX(%s)", ses->nid, ses->nidfmt, ses->sesix?ses->sesix:"");
   
@@ -630,10 +634,10 @@ int zxid_sp_sso_finalize(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n*
   
   if (cf->log_rely_a7n) {
     DD("Logging... %d", 0);
-    logpath = zxlog_path(cf, issuer, a7n->ID, ZXLOG_RELY_DIR, ZXLOG_A7N_KIND, 1);
+    logpath = zxlog_path(cf, issuer, &a7n->ID->g, ZXLOG_RELY_DIR, ZXLOG_A7N_KIND, 1);
     if (logpath) {
       ses->sso_a7n_path = ses->tgt_a7n_path = zx_str_to_c(cf->ctx, logpath);
-      ss = zx_EASY_ENC_WO_sa_Assertion(cf->ctx, a7n);
+      ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &a7n->gg);
       if (zxlog_dup_check(cf, logpath, "SSO assertion")) {
 	if (cf->dup_a7n_fatal) {
 	  err = "C";
@@ -650,11 +654,11 @@ int zxid_sp_sso_finalize(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n*
   zxid_snarf_eprs_from_ses(cf, ses);  /* Harvest attributes and bootstrap(s) */
   cgi->msg = "SSO completed and session created.";
   cgi->op = '-';  /* Make sure management screen does not try to redispatch. */
-  zxid_put_user(cf, ses->nameid->Format, ses->nameid->NameQualifier, ses->nameid->SPNameQualifier, ses->nameid->gg.content, 0);
+  zxid_put_user(cf, &ses->nameid->Format->g, &ses->nameid->NameQualifier->g, &ses->nameid->SPNameQualifier->g, ZX_GET_CONTENT(ses->nameid), 0);
   DD("Logging... %d", 0);
-  zxlog(cf, &ourts, &srcts, 0, issuer, 0, a7n->ID, subj,
+  zxlog(cf, &ourts, &srcts, 0, issuer, 0, &a7n->ID->g, subj,
 	cgi->sigval, "K", "NEWSES", ses->sid, "sesix(%s)", ses->sesix?ses->sesix:"-");
-  zxlog(cf, &ourts, &srcts, 0, issuer, 0, a7n->ID, subj,
+  zxlog(cf, &ourts, &srcts, 0, issuer, 0, &a7n->ID->g, subj,
 	cgi->sigval, "K", ses->nidfmt?"FEDSSO":"TMPSSO", ses->sesix?ses->sesix:"-", 0);
   D_DEDENT("ssof: ");
   return ZXID_SSO_OK;
@@ -662,7 +666,7 @@ int zxid_sp_sso_finalize(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n*
 erro:
   ERR("SSO fail (%s)", err);
   cgi->msg = "SSO failed. This could be due to signature, timeout, etc., technical failures, or by policy.";
-  zxlog(cf, &ourts, &srcts, 0, issuer, 0, a7n?a7n->ID:0, subj,
+  zxlog(cf, &ourts, &srcts, 0, issuer, 0, a7n?&a7n->ID->g:0, subj,
 	cgi->sigval, err, ses->nidfmt?"FEDSSO":"TMPSSO", ses->sesix?ses->sesix:"-", "Error.");
   D_DEDENT("ssof: ");
   return 0;
@@ -696,7 +700,7 @@ int zxid_sp_anon_finalize(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses)
   zxid_snarf_eprs_from_ses(cf, ses);  /* Harvest attributes and bootstrap(s) */
   cgi->msg = "SSO Failure treated as anonymous login and session created.";
   cgi->op = '-';  /* Make sure management screen does not try to redispatch. */
-  /*zxid_put_user(cf, ses->nameid->Format, ses->nameid->NameQualifier, ses->nameid->SPNameQualifier, ses->nameid->gg.content, 0);*/
+  /*zxid_put_user(cf, ses->nameid->Format, ses->nameid->NameQualifier, ses->nameid->SPNameQualifier, ZX_GET_CONTENT(ses->nameid), 0);*/
   zxlog(cf, 0, 0, 0, 0, 0, 0, 0, cgi->sigval, "K", "TMPSSO", "-", 0);
   D_DEDENT("anon_ssof: ");
   return ZXID_SSO_OK;
@@ -738,17 +742,18 @@ int zxid_as_call_ses(zxid_conf* cf, zxid_entity* idp_meta, zxid_cgi* cgi, zxid_s
 
 #if 0
   for (ar_svc = idp_meta->ed->IDPSSODescriptor->ArtifactResolutionService;
-       ar_svc;
+       ar_svc && ar_svc->gg.g.tok == zx_md_ArtifactResolutionService_ELEM;
        ar_svc = (struct zx_md_ArtifactResolutionService_s*)ar_svc->gg.g.n)
     if (ar_svc->Binding  && !memcmp(SAML2_SOAP, ar_svc->Binding->s, ar_svc->Binding->len)
 	/*&& ar_svc->index && !memcmp(end_pt_ix, ar_svc->index->s, ar_svc->index->len)*/
 	&& ar_svc->Location)
       break;
 #else
+  /* *** Kludge: We use the SLO SOAP endpoint for AS. ArtifactResolution might be more natural. */
   for (ar_svc = idp_meta->ed->IDPSSODescriptor->SingleLogoutService;
-       ar_svc;
+       ar_svc && ar_svc->gg.g.tok == zx_md_SingleLogoutService_ELEM;
        ar_svc = (struct zx_md_SingleLogoutService_s*)ar_svc->gg.g.n)
-    if (ar_svc->Binding  && !memcmp(SAML2_SOAP, ar_svc->Binding->s, ar_svc->Binding->len)
+    if (ar_svc->Binding  && !memcmp(SAML2_SOAP, ar_svc->Binding->g.s, ar_svc->Binding->g.len)
 	/*&& ar_svc->index && !memcmp(end_pt_ix, ar_svc->index->s, ar_svc->index->len)*/
 	&& ar_svc->Location)
       break;
@@ -772,11 +777,11 @@ int zxid_as_call_ses(zxid_conf* cf, zxid_entity* idp_meta, zxid_cgi* cgi, zxid_s
   *p = 0;
   ZX_FREE(cf->ctx, buf);
   
-  body = zx_NEW_e_Body(cf->ctx);
-  body->SASLRequest = zx_NEW_as_SASLRequest(cf->ctx);
-  body->SASLRequest->mechanism = zx_dup_str(cf->ctx, "PLAIN");
-  body->SASLRequest->Data = zx_ref_len_simple_elem(cf->ctx, p-b64, b64);
-  r = zxid_soap_call_body(cf, ar_svc->Location, body);
+  body = zx_NEW_e_Body(cf->ctx,0);
+  body->SASLRequest = zx_NEW_as_SASLRequest(cf->ctx, &body->gg);
+  body->SASLRequest->mechanism = zx_dup_attr(cf->ctx, zx_mechanism_ATTR, "PLAIN");
+  body->SASLRequest->Data = zx_ref_len_simple_elem(cf->ctx, &body->SASLRequest->gg, zx_as_Data_ELEM, p-b64, b64);
+  r = zxid_soap_call_body(cf, &ar_svc->Location->g, body);
   /* *** free the body */
   
   if (!r || !r->Envelope || !r->Envelope->Body || !(res = r->Envelope->Body->SASLResponse)) {
@@ -786,17 +791,17 @@ int zxid_as_call_ses(zxid_conf* cf, zxid_entity* idp_meta, zxid_cgi* cgi, zxid_s
     return 0;
   }
   
-  if (!res->Status || !res->Status->code || !res->Status->code->len || !res->Status->code->s) {
+  if (!res->Status || !res->Status->code || !res->Status->code->g.len || !res->Status->code->g.s) {
     ERR("Autentication Service call failed idp(%s). Missing Status code.", idp_meta->eid);
     zxlog(cf, 0,0,0,0,0,0,0, "N", "B", "ERR", 0, "Missing Status code eid(%s)", idp_meta->eid);
     D_DEDENT("as_call: ");
     return 0;
   }
 
-  if (res->Status->code->len != 2
-      || res->Status->code->s[0]!='O' || res->Status->code->s[1]!='K') {  /* "OK" */
-    ERR("Autentication Service call failed idp(%s). Status code(%.*s).", idp_meta->eid, res->Status->code->len, res->Status->code->s);
-    zxlog(cf, 0,0,0,0,0,0,0, "N", "B", "ERR", 0, "Missing Status code(%.*s) eid(%s)", res->Status->code->len, res->Status->code->s, idp_meta->eid);
+  if (res->Status->code->g.len != 2
+      || res->Status->code->g.s[0]!='O' || res->Status->code->g.s[1]!='K') {  /* "OK" */
+    ERR("Autentication Service call failed idp(%s). Status code(%.*s).", idp_meta->eid, res->Status->code->g.len, res->Status->code->g.s);
+    zxlog(cf, 0,0,0,0,0,0,0, "N", "B", "ERR", 0, "Missing Status code(%.*s) eid(%s)", res->Status->code->g.len, res->Status->code->g.s, idp_meta->eid);
     D_DEDENT("as_call: ");
     return 0;
   }
