@@ -14,6 +14,7 @@
  * 26.8.2006, significant Common Subexpression Elimination (CSE) --Sampo
  * 30.9.2007, rework namespaces, esp. previously unknown namespaces  --Sampo
  * 7.10.2008, added documentation --Sampo
+ * 27.10.2010, re-engineered namespace handling --Sampo
  */
 
 #include <string.h>
@@ -21,11 +22,13 @@
 #include "errmac.h"
 #include "zx.h"
 #include "c/zx-data.h"  /* Also generic zx_simple_elem, etc. */
+#include "c/zx-ns.h"
 
 /* ------------- All the manner namespace management ------------- */
 
+#if 0
 /* Called by: */
-void zx_fix_any_elem_dec(struct zx_ctx* c, struct zx_any_elem_s* x, const char* nam, int namlen)
+void zx_fix_any_elem_dec(struct zx_ctx* c, struct zx_elem_s* x, const char* nam, int namlen)
 {
   char* p = memchr(nam, ':', namlen);
   if (p) {
@@ -53,12 +56,41 @@ int zx_is_ns_prefix(struct zx_ns_s* ns, int len, const char* prefix)
       return 1;
   return 0;
 }
+#endif
 
-/* Called by:  zx_fix_any_elem_dec, zx_init_tok_tab, zx_prefix_seen_whine, zx_xmlns_decl, zxsig_validate */
-struct zx_ns_s* zx_locate_ns_by_prefix(struct zx_ctx* c, int len, const char* prefix)
+/*() Debugging function. You can say in gdb: print zx_dump_ns_tab(c,0) */
+
+int zx_dump_ns_tab(struct zx_ctx* c, int flags)
 {
   struct zx_ns_s* ns;
   struct zx_ns_s* alias;
+  int n_ns = c->n_ns;
+  int n=0;
+  for (ns = c->ns_tab; n < n_ns; ++ns) {
+    printf("%3d NS  %8.*s %.*s\n", ++n, ns->prefix_len, ns->prefix, ns->url_len, ns->url);
+    for (alias = ns->n; alias; alias = alias->n)
+      printf("%3d   A %8.*s %.*s\n", ++n, alias->prefix_len, alias->prefix, alias->url_len, alias->url);
+  }
+  for (alias = c->unknown_ns; alias; alias = alias->n) {
+    printf("%3d UNK %8.*s %.*s\n", ++n, alias->prefix_len, alias->prefix, alias->url_len, alias->url);
+  }
+  return n;
+}
+
+/* Profiling of 0.69 on 20101024 shows that 31% of the enc-dec time is spent here.
+ * Surprisingly, the namespace is never found! The only return that executes is 0.
+ * Input was t/azrq1.xml, where in our understanding all prefixes are known.
+ * After fixing a bug related to xmlns declarations being fed to here, the function
+ * was never called. Could we safely delete this? */
+
+/* Called by:  zx_fix_any_elem_dec, zx_init_tok_tab, zx_prefix_seen_whine, zx_xmlns_decl, zxsig_validate */
+static struct zx_ns_s* zx_locate_ns_by_prefix(struct zx_ctx* c, int len, const char* prefix)
+{
+  struct zx_ns_s* ns;
+  struct zx_ns_s* alias;
+  for (ns = c->unknown_ns; ns; ns = ns->n)
+    if (ns->prefix_len == len && (!len || !memcmp(ns->prefix, prefix, len)))
+      return ns;
   for (ns = c->ns_tab; ns->url_len; ++ns) {
     if (ns->prefix_len == len && (!len || !memcmp(ns->prefix, prefix, len)))
       return ns;
@@ -66,50 +98,51 @@ struct zx_ns_s* zx_locate_ns_by_prefix(struct zx_ctx* c, int len, const char* pr
       if (alias->prefix_len == len && (!len || !memcmp(alias->prefix, prefix, len)))
 	return ns;
   }
-  
-  for (alias = c->unknown_ns; alias; alias = alias->n)
-    if (alias->prefix_len == len && (!len || !memcmp(alias->prefix, prefix, len)))
-      return alias;
   return 0;
 }
 
+/* Profiling of 0.69 on 20101024 shows that 64% of the enc-dec time is spent here.
+ * Here the namespace is always found via return ns, but not before searching through
+ * oodles of aliases. After bug fix, this consumes 95% of the zxencdectest run time!
+ * After further bug fixing eliminating scanning (and accumulation) of aliases, the run time went
+ * down by two orders of magnitude. Final solution is the namespace hash lookup. */
+
 /* Called by:  zx_xmlns_decl */
 static struct zx_ns_s* zx_locate_ns_by_url(struct zx_ctx* c, int len, const char* url)
 {
   struct zx_ns_s* ns;
-  struct zx_ns_s* alias;
-  for (ns = c->ns_tab; ns->url_len; ++ns) {
+  for (ns = c->unknown_ns; ns; ns = ns->n)
     if (ns->url_len == len && (!len || !memcmp(ns->url, url, len)))
       return ns;
-    for (alias = ns->n; alias; alias = alias->n)
-      if (alias->url_len == len && (!len || !memcmp(alias->url, url, len)))
-	return alias;
-  }
-
-  for (alias = c->unknown_ns; alias; alias = alias->n)
-    if (alias->url_len == len && (!len || !memcmp(alias->url, url, len)))
-      return alias;
-  return 0;
+  return zx_url2ns(url, len);  /* lookup in perfect hash generated by xsd2sg and gperf */
 }
 
 /* Called by:  zx_prefix_seen_whine, zx_xmlns_decl */
-struct zx_ns_s* zx_new_ns(struct zx_ctx* c, int prefix_len, const char* prefix, int url_len, const char* url)
+static struct zx_ns_s* zx_new_known_ns(struct zx_ctx* c, int prefix_len, const char* prefix, struct zx_ns_s* known_ns)
+{
+  struct zx_ns_s* ns = ZX_ZALLOC(c, struct zx_ns_s);
+  ns->prefix_len = prefix_len;
+  ns->prefix     = prefix;
+  ns->url_len    = known_ns->url_len;
+  ns->url        = known_ns->url;
+  ns->master     = known_ns;
+  return ns;
+}
+
+static struct zx_ns_s* zx_new_unknown_ns(struct zx_ctx* c, int prefix_len, const char* prefix, int url_len, const char* url)
 {
   struct zx_ns_s* ns = ZX_ZALLOC(c, struct zx_ns_s);
   ns->prefix_len = prefix_len;
   ns->url_len = url_len;
-#if 0
-  ns->prefix = prefix;
-  ns->url = url;
-#else
-  /* *** reallocating these should not be necessary, except for unknown namespaces */
+  /* Reallocating these is necessary for unknown namespaces */
   ns->prefix = ZX_ALLOC(c, prefix_len+1);
+  if (prefix_len)
+    memcpy((char*)ns->prefix, prefix, prefix_len);
+  *((char*)ns->prefix+prefix_len) = 0;
   ns->url = ZX_ALLOC(c, url_len+1);
-  memcpy(ns->prefix, prefix, prefix_len);
-  memcpy(ns->url, url, url_len);
-  ns->prefix[prefix_len] = 0;
-  ns->url[url_len] = 0;
-#endif
+  if (url_len)
+    memcpy((char*)ns->url, url, url_len);
+  *((char*)ns->url+url_len) = 0;
   return ns;
 }
 
@@ -121,32 +154,50 @@ struct zx_ns_s* zx_new_ns(struct zx_ctx* c, int prefix_len, const char* prefix,
 static struct zx_ns_s* zx_xmlns_decl(struct zx_ctx* c, int prefix_len, const char* prefix, int url_len, const char* url)
 {
   struct zx_ns_s* alias;
+  struct zx_ns_s* new_ns;
   struct zx_ns_s* ns;
 
-  /* Always alloc a new one because we may need to push to stack multiple instances of same ns. */
-  alias = zx_new_ns(c, prefix_len, prefix, url_len, url);
-
   ns = zx_locate_ns_by_url(c, url_len, url);
   if (!ns) {
-    D("Namespace(%.*s) not found by URL(%.*s) (probably wrong namespace URL)", prefix_len, prefix, url_len, url);
+    D("Namespace(%.*s) not found by URL(%.*s) (probably unknown or wrong namespace URL)", prefix_len, prefix, url_len, url);
+    zx_xml_parse_dbg(c, '-', __FUNCTION__, "Namespace here");
     ns = zx_locate_ns_by_prefix(c, prefix_len, prefix);
     if (!ns) {
       /* Namespace not known by compiled in schemata. */
-      D("Namespace not found by prefix(%.*s) or URL(%.*s) (probably wrong namespace URL)", prefix_len, prefix, url_len, url);
-      alias->n = c->unknown_ns;
-      c->unknown_ns = alias;
-      return alias;
+      D("Namespace not found by prefix(%.*s) or URL(%.*s) (probably unknown or wrong namespace URL)", prefix_len, prefix, url_len, url);
+      new_ns = zx_new_unknown_ns(c, prefix_len, prefix, url_len, url);
+      new_ns->n = c->unknown_ns;
+      c->unknown_ns = new_ns;
+      return new_ns;
     }
   }
-  alias->n = ns->n;
-  ns->n = alias;
-  return alias;
+  /* Always alloc a new one because we may need to push to stack multiple instances of same ns. */
+  new_ns = zx_new_known_ns(c, prefix_len, prefix, ns);
+
+  /* Avoid adding to alias list if prefix is already known. */
+  if (ns->prefix_len == prefix_len && (!prefix_len || !memcmp(ns->prefix, prefix, prefix_len)))
+    goto known_prefix;
+  for (alias = ns->n; alias; alias = alias->n)
+    if (alias->prefix_len == prefix_len && (!prefix_len || !memcmp(alias->prefix, prefix, prefix_len)))
+      goto known_prefix;
+  
+  D("New prefix(%.*s) known URL(%.*s)", prefix_len, prefix, url_len, url);
+  ns->n = new_ns->n;
+  new_ns->n = ns;
+known_prefix:
+  return new_ns;
 }
 
+/*() Scan the seen list (expected to be relatively short so scan
+ * is not a perfomance issue). The scan walks the doubly linked
+ * list of currently active seen prefixes (as opposed to seen_pop). */
+
 /* Called by:  zx_prefix_seen_whine, zx_push_seen */
 struct zx_ns_s* zx_prefix_seen(struct zx_ctx* c, int len, const char* prefix)
 {
   struct zx_ns_s* ns;
+  if (len == 3 && !memcmp(prefix, "xml", 3))
+      return zx_ns_tab + (zx_xml_NS >> ZX_TOK_NS_SHIFT);
   for (ns = c->guard_seen_n.seen_n; ns->seen_n; ns = ns->seen_n)
     if (ns->prefix_len == len && (!len || !memcmp(ns->prefix, prefix, len)))
       return ns;
@@ -154,7 +205,7 @@ struct zx_ns_s* zx_prefix_seen(struct zx_ctx* c, int len, const char* prefix)
 }
 
 /*() zx_prefix_seen_whine() is a good place to detect, and add stub for,
- * wholly unknown prefixes. */
+ * wholly unknown prefixes. Seldom returns NULL, just always makes something up. */
 
 /* Called by:  TXattr_lookup, TXelem_lookup */
 struct zx_ns_s* zx_prefix_seen_whine(struct zx_ctx* c, int len, const char* prefix, const char* logkey, int mk_dummy_ns)
@@ -168,13 +219,14 @@ struct zx_ns_s* zx_prefix_seen_whine(struct zx_ctx* c, int len, const char* pref
     if (!ns) {
       if (mk_dummy_ns) {
 	url = zx_strf(c, "urn:zxid:unknown-ns-prefix:%s:%.*s", logkey, len, STRNULLCHK(prefix));
-	ns = zx_new_ns(c, len, prefix, url->len, url->s);
+	ns = zx_new_unknown_ns(c, len, prefix, url->len, url->s);
 	ns->n = c->unknown_ns;
 	c->unknown_ns = ns;
 	D("Undefined namespace prefix(%.*s). NS not known from any context. Creating dummy ns(%.*s).", len, prefix, url->len, url->s);
 	ZX_FREE(c, url);
       } else {
-	DD("Undefined namespace prefix(%.*s) at(%s). NS not known from any context.", len, prefix, logkey);
+	D("Undefined namespace prefix(%.*s) at(%s). NS not known from any context.", len, prefix, logkey);
+	return 0;
       }
     } else {
       if (len != 3 || memcmp(prefix, "xml", 3)) {
@@ -232,6 +284,8 @@ void zx_pop_seen(struct zx_ns_s* ns)
   }
 }
 
+/*() Collect namespaces from element */
+
 void zx_see_elem_ns(struct zx_ctx* c, struct zx_ns_s** pop_seen, struct zx_elem_s* el)
 {
   struct zx_ns_s* ns;
@@ -252,6 +306,7 @@ void zx_see_elem_ns(struct zx_ctx* c, struct zx_ns_s** pop_seen, struct zx_elem_
  * a lousy design. In order to handle this we need to either backtrack or
  * make a special case forward scan for xmlns attributes (which is redundant with
  * normal attribute scanning). It seems simpler to do the latter, so here goes...
+ * (besides, the forward scan will prime the cache)
  *
  * The return value represents the list of namespaces that were newly declared
  * at this level, i.e. pushed to the seen stacks. This list is used to pop the
@@ -320,6 +375,7 @@ struct zx_ns_s* zx_scan_xmlns(struct zx_ctx* c)
   return pop_seen;
 }
 
+#if 0  /* schedule to remove */
 /*() Disambiguate token by considering its namespace.
  * See zx_attr_lookup(), zx_elem_lookup()
  * For attributes the namespaceless case is considered. */
@@ -353,6 +409,7 @@ const struct zx_tok* zx_tok_by_ns(const struct zx_tok* zt, const struct zx_tok*
 	return zt;
   return ztt;
 }
+#endif
 
 #if 0
 /*(-) N.B. Generally this function is not needed since tokens can be arranged
@@ -372,43 +429,23 @@ int zx_init_tok_tab(struct zx_ctx* c, struct zx_tok* tok_tab, struct zx_tok* tok
 #endif
 
 /* Called by:  zx_len_inc_ns */
-int zx_len_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_s** pop_seen)
+int zx_len_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_s** pop_seenp)
 {
   if (!ns)
     return 0;
-  if (!zx_push_seen(c, ns->prefix_len, ns->prefix, ns->url_len, ns->url, pop_seen))
+  if (!zx_push_seen(c, ns->prefix_len, ns->prefix, ns->url_len, ns->url, pop_seenp))
     return 0;
   return sizeof(" xmlns")-1
     + (ns->prefix_len ? ns->prefix_len+1 : 0) + 2 + ns->url_len + 1;
 }
 
-/* Called by:  zx_enc_inc_ns */
-char* zx_enc_xmlns_if_not_seen(struct zx_ctx* c, char* p, struct zx_ns_s* ns, struct zx_ns_s** pop_seen)
-{
-  if (!ns)
-    return p;
-  if (!zx_push_seen(c, ns->prefix_len, ns->prefix, ns->url_len, ns->url, pop_seen))
-    return p;
-  
-  ZX_OUT_MEM(p, " xmlns", sizeof(" xmlns")-1);
-  if (ns->prefix_len) {
-    ZX_OUT_CH(p, ':');
-    ZX_OUT_MEM(p, ns->prefix, ns->prefix_len);
-  }
-  ZX_OUT_CH(p, '=');
-  ZX_OUT_CH(p, '"');
-  ZX_OUT_MEM(p, ns->url, ns->url_len);
-  ZX_OUT_CH(p, '"');
-  return p;
-}
-
 /*() For WO encoder the order of xmlns declarations is not known at compile
  * time. Thus we first add them to the pop_seen list using insertion
  * sort (pop_seen is smallest and prefixes grow from there) and
  * then later render the list using zx_enc_seen(). */
 
 /* Called by:  TXENC_WO_any_elem, zx_add_inc_ns */
-void zx_add_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_s** pop_seen)
+void zx_add_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_s** pop_seenp)
 {
   struct zx_ns_s* pop_seen_dummy=0;
   struct zx_ns_s* new_ns;
@@ -418,18 +455,18 @@ void zx_add_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns
   new_ns = zx_push_seen(c, ns->prefix_len, ns->prefix, ns->url_len, ns->url, &pop_seen_dummy);
   if (!new_ns)
     return;
-  if (!*pop_seen) {
-    *pop_seen = new_ns;
+  if (!*pop_seenp) {
+    *pop_seenp = new_ns;
     return;
   }
   if (!new_ns->prefix_len) {       /* Default namespace (empty prefix) sorts first. */
 first:
-    new_ns->seen_pop = *pop_seen;
-    *pop_seen = new_ns;
+    new_ns->seen_pop = *pop_seenp;
+    *pop_seenp = new_ns;
     return;
   }
 
-  ns = *pop_seen;
+  ns = *pop_seenp;
   if (ns->prefix_len) {
     res = memcmp(ns->prefix, new_ns->prefix, MIN(ns->prefix_len, new_ns->prefix_len));
     if (res > 0 || !res && ns->prefix_len >= new_ns->prefix_len)
@@ -445,10 +482,11 @@ first:
   ns->seen_pop = new_ns;
 }
 
+/*() Render the namespaces that have been seen. The list is assumed to be already sorted. */
+
 /* Called by:  TXENC_WO_ELNAME, TXENC_WO_any_elem */
 char* zx_enc_seen(char* p, struct zx_ns_s* ns)
 {
-  /* *** Consider sorting the seen */
   for (; ns; ns = ns->seen_pop) {
     ZX_OUT_MEM(p, " xmlns", sizeof(" xmlns")-1);
     if (ns->prefix_len) {
@@ -463,4 +501,34 @@ char* zx_enc_seen(char* p, struct zx_ns_s* ns)
   return p;
 }
 
+/*() dec-templ.c CSE elimination. */
+
+struct zx_ns_s* zx_xmlns_detected(struct zx_ctx* c, struct zx_elem_s* x, const char* url)
+{
+  struct zx_ns_s* ns;
+  const char* prefix;
+  /* Scan backwards to find beginning of name. Already cached. Scan is cheaper than arg. */
+  for (prefix = url-2; !ONE_OF_5(*prefix, ':', ' ', '\n', '\r', '\t'); --prefix) ;
+  ++prefix;
+  DD("xmlns detected(%.*s)", url-2-prefix, prefix);
+  ns = ZX_ZALLOC(c, struct zx_ns_s);
+  ns->url = url;
+  ns->url_len = c->p - url;
+  ns->prefix_len = url-2-prefix;
+  ns->prefix = prefix;
+  ns->n = x->xmlns;
+  x->xmlns = ns;
+  return ns;
+}
+
+/* These do not have their own elems, so the table andlookup were not generated. Supply dummies. */
+
+struct zx_el_tok zx_xs_el_tab[zx_xs__ELEM_MAX] = { {""} };
+struct zx_el_tok zx_xsi_el_tab[zx_xsi__ELEM_MAX] = { {""} };
+struct zx_el_tok* zx_xs_elem2tok(const char* str, unsigned int len) { return 0; }
+struct zx_el_tok* zx_xsi_elem2tok(const char* str, unsigned int len) { return 0; }
+
+//struct zx_el_tok zx_xml_el_tab[zx_xml__ELEM_MAX] = { {""} };
+//struct zx_el_tok* zx_xml_elem2tok(const char* str, unsigned int len) { return 0; }
+
 /* EOF -- zxns.c */
